package hr.fer.zemris.projekt.algorithm;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Predstavlja jedan algoritam. Sadrži podatke o njegovom nazivu, tipu, stazi do vezanog vanjskog
 * izvršnog programa te vezanom procesu koji ga izvodi. Nudi metode za pokretanje, čekanje na
 * završetak te prisilno završavanje izvođenja algoritma.
 * 
 * @author
 * 
 */
public class Algorithm {

    /** Naziv algoritma */
    private String name;
    /** Tip algoritma */
    private String type;
    /** Staza do vanjskog izvršnog programa vezanog za algoritam */
    private String binaryPath;
    /** Proces trenutno vezan za izvršavanje algoritma */
    private Process process = null;
    /** Naziv zadatka */
    private String problemName;
    
    /**
     * Konstruktor Algorithm.
     * 
     * @param name naziv algoritma
     * @param type tip algoritma
     * @param binaryPath izvršna staza do vanjskog programa algoritma
     * @throws IOException ukoliko izvršna staza ne pokazuje na izvršnu datoteku
     */
    public Algorithm(String name, String type, String binaryPath) throws IOException {
		File f = new File(binaryPath);
		if (!f.canExecute()) {
		    throw new IOException("'" + binaryPath + "' does not exist or is not executable.");
		}
	
		this.name = name;
		this.type = type;
		this.binaryPath = binaryPath;
    }

    /**
     * Pokreće izvođenje algoritma izvršavanjem vanjske izvršne datoteke vezane za algoritam.
     * 
     * @throws IOException ukoliko je nemoguće pokrenuti vanjski izvršni program
     */
    public void run() throws IOException {
		Runtime r = Runtime.getRuntime();
		try {
			process = r.exec(binaryPath + " " + problemName);
		    
		} catch (IOException e) {
		    throw new IOException("'" + binaryPath + "' could not be run.");
		}
    }

    /**
     * Blokiranjem dretve čeka da proces vezan za izvođenje algoritma završi. Ukoliko se algoritam
     * trenutno ne izvodi ne događa se ništa.
     */
    public void waitFor() {
		if (process == null) {
		    return;
		}
	
		int done = -666;
		while (done == -666) {
		    try {
			done = process.waitFor();
			} catch (InterruptedException e) {
		    	e.printStackTrace();
		    }
		}
	
		process = null;
    }

    /**
     * Trenutno prekida izvršavanje procesa vezanog za algoritam. Ukoliko se algoritam trenutno ne
     * izvodi ne događa se ništa.
     */
    public void kill() {
		if (process == null) {
		    return;
		}
		process.destroy();
    }

    /**
     * Dohvaća posljednji rezultat algoritma iz odgovarajuće izlazne datoteke.
     * 
     * @return posljednji rezultat algoritma
     */
    public AlgorithmResult getLatestResult() {
		AlgorithmResult rezultat = new AlgorithmResult(this);
		try {
			BufferedReader citac = new BufferedReader(new FileReader("rjesenja\\" + name + "_" + problemName + ".out"));
			String red = citac.readLine();
			rezultat.setTipMaterijala(red);
			String sawWidth = citac.readLine();
			red = citac.readLine();
			rezultat.setGoodCut(Integer.parseInt(red));
			ArrayList<String> rezultatBuffer = new ArrayList<String>();
			red = citac.readLine();
			
			while(red != null){
				red = citac.readLine();
				rezultatBuffer.clear();
				while(red != null && !red.equals(new String("##########"))){
					rezultatBuffer.add(red);
					red = citac.readLine();
				}
			}
			citac.close();
			Pattern stock = Pattern.compile("(.+)\t");
			Pattern Itemi = Pattern.compile("(.+?)\\+");
			Pattern input = Pattern.compile(".+");
			Matcher m;
			for(String i : rezultatBuffer){
				m = stock.matcher(i);
				m.find();
				Layout plan = new Layout(Integer.parseInt(m.group(1)), Integer.parseInt(sawWidth));
				m.usePattern(input);
				while(m.find()){
					m = Itemi.matcher(m.group());
					if (!m.find()){
						break;
					}
					plan.addItem(Integer.parseInt(m.group(1)));
					m.usePattern(input);
				}
				
				rezultat.addLayout(plan);
			}
		} catch (FileNotFoundException e) {
			System.out.println("Neuspješno otvaranje izlazne datoteke algoritma " + name);
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("Neuspješno citanje iz izlazne datoteke algoritma " + name);
			e.printStackTrace();
		}
		return rezultat;
    }

    /**
     * Dohvaća naziv algoritma.
     * 
     * @return naziv algoritma
     */
    public String getName() {
    	return name;
    }

    /**
     * Dohvaća tip algoritma.
     * 
     * @return tip algoritma
     */
    public String getType() {
    	return type;
    }

    /**
     * Dohvaća izvršnu stazu algoritma.
     * 
     * @return izvršna staza algoritma
     */
    public String getBinaryPath() {
    	return binaryPath;
    }
    
    /**
     * Postavlja naziv zadatka
     * 
     * @param problemName naziv zadatka
     */
	public void setProblemName(String problemName) {
		this.problemName = problemName;
	}
}
