package hr.fer.zemris.projekt.algorithm;

import hr.fer.zemris.projekt.database.DatabaseProvider;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JTable;

/**
 * Razred predstavlja problem koji algoritam rješava. Sadrži ključne podatke koje definiraju problem
 * te funkcionalnost za provjeru ispravnosti tih podataka. Također nudi ispis problema u vanjsku
 * datoteku za korištenje algoritmu ili za kasnije konstruiranje jednakog novog problema. Može se
 * konstruirati iz datoteke stvorene iz prošle instance problema ili iz podataka iz glavnog
 * korisničkog sučelja.
 * 
 * @author
 * 
 */
public class AlgorithmProblem {

    /** Raspoložive sirovine */
    private Integer[][] stock;
    /** Željeni rezovi */
    private Map<Integer, Integer> items;
    /** Mjera za dovoljno dobar rez, ukoliko je unutar te granice rez se smatra savršenim */
    private int goodEnoughCut;
    /** Debljina pile tj. gubitak materijala pri jednom rezu */
    private int sawWidth;
    /** Tip sirovina korištenih u ovom problemu */
    private String stockType;
    /** Naziv problema */
    private String problemName;
    /** Maksimalno vrijeme izvođenja */
    private int maxTime;
    
    /**
     * Konstruktor AlgorithmProblem.
     * 
     * @param stockTable tablica sirovina iz glavnog prozora, predstavlja trenutno stanje
     *            raspoloživih sirovina
     * @param itemTable tablica željenih rezova iz glavnog prozora
     * @param goodEnoughCut mjera za dovoljno dobar rez, ukoliko je unutar te granice rez se smatran
     *            savršenim
     * @param sawWidth debljina pile tj. gubitak materijala pri jednom rezu
     * @param stockType tip sirovina korištenih u ovom problemu
     * @param problemName naziv problema
     */
    public AlgorithmProblem(JTable stockTable, JTable itemTable, int goodEnoughCut, int sawWidth,
	    String stockType, String problemName, int maxTime) {
		items = new HashMap<Integer, Integer>();
		fillItems(itemTable);
		fillStock(stockTable);
		this.goodEnoughCut = goodEnoughCut;
		this.sawWidth = sawWidth;
		this.stockType = stockType;
		this.problemName = problemName;
		this.maxTime = maxTime;
    }
    
    /**
     * Konstruktor AlgorithmProblem. Koristi ulaznu datoteku pri stvaranju objekta. Namjenjen za
     * korištenje s datotekama stvorenim metodom exportToProblemSaveFile().
     * 
     * @param f ulazna datoteka s problemom
     */
    public AlgorithmProblem(File f) {
		stock = null;
		items = new HashMap<Integer, Integer>();
		try {
			Pattern p = Pattern.compile("^(.+)\\.(.+)");
			Matcher m = p.matcher(f.getName());
			if (!m.find()){
				throw new IOException();
			}
			problemName = m.group(1);
			String ekstenzija = m.group(2);
			BufferedReader citac = new BufferedReader(new FileReader(f));
			if (ekstenzija.equals("sav")){
				String red = citac.readLine();
				sawWidth = Integer.parseInt(red);
				red = citac.readLine();
				goodEnoughCut = Integer.parseInt(red);
				red = citac.readLine();
				stockType = red;
				red = citac.readLine();
				while(red != null){
					String[] temp = red.split("\t");
					items.put(new Integer(temp[0]), new Integer(temp[1]));
					red = citac.readLine();
				}
			} else {
				String red = citac.readLine();
				stockType = red;
				Object[][] typeData = DatabaseProvider.getDatabase().getStockTypeData();
				int sirina = -1;
				for(Object[] i : typeData){
					if (stockType.equals(i[0].toString())){
						sirina = Integer.parseInt(i[1].toString());
						break;
					}
				}
				if (sirina == -1){
					stockType = null;
					return;
				}
				red = citac.readLine();
				while(red != null && red.charAt(0) != '#'){
					String[] temp = red.split("\t");
					Integer key = new Integer(temp[0]);
					Integer value = new Integer(temp[1]);
					if (items.containsKey(key)){
						Integer sum = items.get(key) + value;
						items.put(key, sum);
					} else {
						items.put(key, value);
					}
					red = citac.readLine();
				}
				if (red == null){
					return;
				}
				red = citac.readLine();
				while(red != null){
					String[] temp = red.split("\t");
					int x = Integer.parseInt(temp[0]);
					int y = Integer.parseInt(temp[1]);
					int broj = Integer.parseInt(temp[2]);
					y = (int) Math.ceil((double)y/sirina);
					items.put(new Integer(x), new Integer(y*broj));
					red = citac.readLine();
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    /**
     * Strukturu stock puni podacima iz tablice sirovina glavnog prozora, tj. raspoloživim
     * sirovinama ovog problema. Namjenjena za korištenje u verziji konstruktora koja zadatak
     * učitava iz glavnog sučelja.
     * 
     * @param stockTable tablica sirovina iz glavnog prozora, predstavlja trenutno stanje
     *            raspoloživih sirovina
     */
    private void fillStock(JTable stockTable) {
		int r = stockTable.getRowCount();
		stock = new Integer[r][2];
		for (int i = 0; i < r; ++i) {
			stock[i][0] = (Integer) stockTable.getValueAt(i, 0);
	    	stock[i][1] = (Integer) stockTable.getValueAt(i, 1);
		}
    }

    /**
     * Strukturu items puni podacima iz tablice željenih rezova glavnog prozora. Namjenjena za
     * korištenje u verziji konstruktora koja zadatak učitava iz glavnog sučelja.
     * 
     * @param itemTable tablica željenih rezova iz glavnog prozora
     */
    private void fillItems(JTable itemTable) {
		int r = itemTable.getRowCount();
		for (int i = 0; i < r; ++i) {
			items.put((Integer)itemTable.getValueAt(i, 0), (Integer)itemTable.getValueAt(i, 1));
		}
    }

    /**
     * Provjerava ispravnost problema.
     * 
     * @return true ako je problem ispravan, false ako nije
     */
    public boolean isValid() {
		int maxStock = 0;
		for(Integer[] i : stock){
			if (i[1] == Integer.MAX_VALUE && i[0] > maxStock){
				maxStock = i[0];
			}
		}
    	for(Integer i : items.keySet()){
    		if (i.intValue() > maxStock){
    			return false;
    		}
    	}
    	if (items.size() == 0){
    		return false;
    	}
		return true;
    }

    /**
     * Problem ispisuje u izlaznu datoteku. Tako stvorena datoteka je namjenjena za kasnije
     * učitavanje iz eksternih programa algoritama.
     */
    public void exportToAlgorithmInputFile() {
    	try {
			File f = new File("algoritmi\\" + problemName + ".in");
    		if (f.exists()){
    			f.delete();
    		}
			FileWriter ispis = new FileWriter(f);
    		ispis.write(String.valueOf(maxTime) + "\t" + String.valueOf(sawWidth) + "\t" + String.valueOf(goodEnoughCut) + "\n");
    		int minItem = Integer.MAX_VALUE;
    		for (Integer i : items.keySet()){
    			if (i.intValue() < minItem){
    				minItem = i.intValue();
    			}
    		}
    		for(Integer[] i : stock){
				if (i[0].intValue() < minItem){
					continue;
				}
    			ispis.write(i[0].toString() + "\t");
				if (i[1].intValue() == Integer.MAX_VALUE){
					ispis.write("-1\n");
				} else {
					ispis.write(i[1].toString() + "\n");
				}
			}
			ispis.write("----------\n");
			for(Integer i : items.keySet()){
				ispis.write(i.toString() + "\t" + items.get(i).toString() + "\n");
			}
			ispis.flush();
			ispis.close();
		} catch (IOException e) {
			System.out.println("Greška prilikom otvaranja ulazne datoteke algoritama");
			e.printStackTrace();
		}
    }

    /**
     * Problem ispisuje u izlaznu datoteku. Tako stvorena datoteka je namjenjena za kasnije ponovno
     * učitavanje konstruktorom AlgorithmProblem(File f).
     */
    public void exportToProblemSaveFile(File f) {
    	f.setWritable(true, false);
		try {
			FileWriter ispis = new FileWriter(f);
			ispis.write(String.valueOf(sawWidth) + "\n");
			ispis.write(String.valueOf(goodEnoughCut) + "\n");
			ispis.write(stockType + "\n");
			for(Integer i : items.keySet()){
				ispis.write(i.toString() + "\t" + items.get(i).toString() + "\n");
			}
			ispis.flush();
			ispis.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    /**
     * Dohvaća raspoložive sirovine.
     * 
     * @return raspoložive sirovine
     */
    public Integer[][] getStock() {
    	return stock;
    }

    /**
     * Postavlja raspoložive sirovine.
     * 
     * @param stock raspoložive sirovine
     */
    public void setStock(Integer[][] stock) {
    	this.stock = stock;
    }

    /**
     * Dohvaća željene rezove.
     * 
     * @return željeni rezovi
     */
    public Integer[][] getItems() {
    	int redovi = this.items.size();
    	Integer[][] items = new Integer[redovi][2];
    	int k = 0;
    	for(Integer i : this.items.keySet()){
    		items[k][0] = i;
    		items[k++][1] = this.items.get(i);
    	}
    	return items;
    }

    /**
     * Postavlja željene rezove.
     * 
     * @param items željeni rezovi
     */
    public void setItems(Integer[][] items) {
    	this.items = new HashMap<Integer, Integer>();
    	for (Integer[] i : items) {
			this.items.put(i[0], i[1]);
		}
    }

    /**
     * Dohvaća mjeru za dovoljno dobar rez.
     * 
     * @return mjera za dovoljno dobar rez
     */
    public int getGoodEnoughCut() {
    	return goodEnoughCut;
    }

    /**
     * Postavlja mjeru za dovoljno dobar rez.
     * 
     * @param goodEnoughCut mjera za dovoljno dobar rez
     */
    public void setGoodEnoughCut(int goodEnoughCut) {
    	this.goodEnoughCut = goodEnoughCut;
    }

    /**
     * Dohvaća debljinu pile tj. gubitak materijala pri jednom rezu.
     * 
     * @return debljina pile tj. gubitak materijala pri jednom rezu
     */
    public int getSawWidth() {
    	return sawWidth;
    }

    /**
     * Postavlja debljinu pile tj. gubitak materijala pri jednom rezu.
     * 
     * @param sawWidth debljina pile tj. gubitak materijala pri jednom rezu
     */
    public void setSawWidth(int sawWidth) {
    	this.sawWidth = sawWidth;
    }

    /**
     * Dohvaća tip sirovine.
     * 
     * @return tip sirovine
     */
    public String getStockType() {
    	return stockType;
    }

    /**
     * Postavlja tip sirovine.
     * 
     * @param stockType tip sirovine
     */
    public void setStockType(String stockType) {
    	this.stockType = stockType;
    }

    /**
     * Dohvaća naziv problema.
     * 
     * @return naziv problema
     */
    public String getProblemName() {
    	return problemName;
    }

    /**
     * Postavlja naziv problema.
     * 
     * @param problemName naziv problema
     */
    public void setProblemName(String problemName) {
    	this.problemName = problemName;
    }
    
    /**
     * Dohvaća maksimalno vrijeme izvođenja
     * 
     * @return maksimalno vrijeme izvođenja
     */
	public int getMaxTime() {
		return maxTime;
	}
	
	/**
     * Postavlja maksimalno vrijeme izvođenja
     * 
     * @param maxTime maksimalno vrijeme izvođenja
     */
	public void setMaxTime(int maxTime) {
		this.maxTime = maxTime;
	}
    
}
