package hr.fer.zemris.projekt.algorithm;

import hr.fer.zemris.projekt.database.Database;
import hr.fer.zemris.projekt.database.DatabaseProvider;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Razred predstavlja jedan rezultat tj. rješenje algoritma. Sadrži podatke i metode potrebne za
 * iscrtavanje grafičke reprezentacije rješenja te za aktualizaciju rješenja u sustavu i bazi
 * podataka.
 * 
 * @author
 * 
 */
public class AlgorithmResult implements Iterable<Layout> {
	/** lista planova rezanja */
	private List<Layout> layouts;
	/** ukupna duljina ostataka */
	private int totalRemainder;
	/** ukupna duljina materijala */
	private int totalMaterial;
	/** duljina najduljeg materijala */
	private int longest;
	/** popis korištenih materijala */
	private Map<Integer, Integer> popisMaterijala;
	/** tip korištenog materijala */
	private String tipMaterijala;
	/** algoritam koji je stvorio rješenje */
	private Algorithm algoritam;
	/** ukupna duljina ostataka bez piljevine*/
	private int totalWaste;
	/** doljnja granica duljine stockova koje spremamo u bazu poslije izvođenja programa*/
	private int goodCut;
	
	/**
     * Konstruktor AlgorithmResult.
     */
    public AlgorithmResult(Algorithm roditelj) {
    	this.layouts = new LinkedList<Layout>();
    	this.totalRemainder = 0;
    	this.totalMaterial = 0;
    	this.longest = 0;
    	this.popisMaterijala = new HashMap<Integer, Integer>();
    	this.algoritam = roditelj;
    }

    /**
     * Aktualizira rješenje u sustavu i bazi podataka. Praktično oduzima korištene sirovine i
     * ispisuje rezultat.
     * @param f .txt datoteka u koju se rješenje ispisuje
     * @param flag boolean vrijednost da li se zapisuju ostatci materijala u bazu
     */
    public void actualize(File f, boolean flag) {
    	Database baza = DatabaseProvider.getDatabase();
    	for (Integer i : popisMaterijala.keySet()) {
			baza.removeStock(i.intValue(), popisMaterijala.get(i).intValue(), tipMaterijala);
		}
    	try {
    		f.setWritable(true, false);
    		FileWriter ispis = new FileWriter(f);
			for(Layout i : layouts){
	    		ispis.write(i.toString() + "\n");
	    		if (flag && i.getRemainder() > goodCut){
	    			Object[][] stockovi = baza.getStockData();
	    			int kolicina = 1;
	    			for(Object[] j : stockovi){
	    				if (((Integer)j[0] == i.getRemainder()) && j[2].toString().equals(tipMaterijala)){
	    					kolicina += Integer.parseInt(j[1].toString());
	    					baza.removeStock(i.getRemainder(), Integer.parseInt(j[1].toString()), tipMaterijala);
	    				}
	    			}
	    			baza.insertStock(i.getRemainder(), kolicina, tipMaterijala);
	    		}
	    	}
	    	ispis.flush();
	    	ispis.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    /**
     * Metoda koja dodaje plan rezanja jednog materijala u ovo rješenje. 
     * @param layout plan rezanja jednog materijala
     * @return true ako je dodavanje uspjelo, false inače
     */
    public boolean addLayout(Layout layout) {
    	if (layouts.add(layout)) {
    		totalRemainder += layout.getRemainder();
    		totalMaterial += layout.getStockSize();
    		totalWaste += layout.getWaste();
    		if (popisMaterijala.containsKey(layout.getStockSize())){
    			popisMaterijala.put(layout.getStockSize(), popisMaterijala.get(layout.getStockSize())+1);
    		} else {
    			popisMaterijala.put(layout.getStockSize(), 1);
    		}
    		if (longest < layout.getStockSize()) {
    			longest = layout.getStockSize();
    		}
    		return true;
    	}
    	return false;
    }
    
    /**
	 * Metoda koja postavlja vrijednost varijable tipMaterijala (potrebno za metodu actualize())
	 */
	public void setTipMaterijala(String tip) {
		this.tipMaterijala = tip;
	}

	/**
	 * Metoda koja vraća iterator po listi planova rezanja(layouts). Nije 
	 * predviđeno da se pomoću danog iteratora mijenja lista ili njezini elementi.
	 * @return iterator po listi planova rezanja
	 */
	public Iterator<Layout> iterator() {
		return layouts.iterator();
	}

    /**
     * Metoda koja vraća ukupnu količinu (duljinu) materijala iskorištenu. 
     * @return ukupuna iskorištena količina materijala
     */
	public int getTotalMaterial() {
		return totalMaterial;
	}

	/**
	 * Metoda koja vraća ukupnu količinu otpada (bez uračunatog gubitka pri rezanju).
	 * @return ukupna količina otpada
	 */
	public int getTotalRemainder() {
		return totalRemainder;
	}
	
	/**
	 * Metoda koja vraća ukupnu količinu otpada (sa uračunatim gubicima pri rezanju).
	 * @return ukupna količina otpada
	 */
	public int getTotalWaste() {
			return totalWaste;
	}
	/**
	 * Metoda koja vraća duljinu najduljeg upotrebljenog materijala.  
	 * @return duljina najduljeg upotrebljenog materijala
	 */
	public int getLongest() {
		return longest;
	}
	
	/**
	 * Metoda koja vraća broj upotrebljenih materijala. 
	 * @return broj materijala upotrebljen
	 */
	public int numOfLayouts() {
		return layouts.size();
	}
	
	/**
	 * Metoda koja vraća podatke o algoritmu koji je stvorio ovo rješenje. 
	 * @return algoritam koji je stvorio ovo rješenje
	 */
	public Algorithm getAlgorithm() {
		return algoritam;
	}
	
	/**
	 * Metoda koja vraća najmanju duljinu koja se sprema u bazu. 
	 * @return najmanja duljina koja se sprema u bazu
	 */
	public int getGoodCut() {
		return goodCut;
	}
	
	/**
	 * Metoda koja postavlja najmanju duljinu koja se sprema u bazu. 
	 * @param goodCut najmanja duljina koja se sprema u bazu
	 */
	public void setGoodCut(int goodCut) {
		this.goodCut = goodCut;
	}
	
}
