package hr.fer.zemris.projekt.algorithm;

import hr.fer.zemris.projekt.swing.MainWindow;
import hr.fer.zemris.projekt.util.Config;

import java.awt.Toolkit;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JProgressBar;
import javax.swing.SwingWorker;

/**
 * Razred predstavlja zadaću pokretanja niza algoritama, praćenja i prekidanja njihovog izvođenja.
 * Također nudi funkcionalnost obavještavanja zainteresiranih slušača o dostupnosti rezultata i
 * ažuriranja glavnog korisničkog sučelja s grafičkim indikatorom napretka.
 * 
 * @author
 * 
 */
public class AlgorithmRunnerTask extends SwingWorker<Void, AlgorithmResult> {

    /** Trenutno izvršavana zadaća, u biti jedina instanca ovog razreda */
    public static AlgorithmRunnerTask currentlyRunningTask = null;

    /** Slušači na dostupnost rezultata algoritama */
    private List<AlgorithmResultsListener> algorithmResultsListeners;
    /** Lista algoritama koji se pokreću */
    private List<Algorithm> algorithmList;
    /** Broj izvršavanja pojedinog algoritma */
    private int numberOfRuns;
    /** Glavni prozor korisničkog sučelja */
    private MainWindow mainWindow;
    /** Grafički indikator napretka ukupne zadaće */
    private JProgressBar progressBar;
    /** Gumb za pokretanje izvršavanja zadaće */
    private JButton startButton;
    /** Gumb za prekid izvršavanja zadaće */
    private JButton stopButton;
    /** Trenutno izvšravan algoritam u okviru zadaće */
    private Algorithm currentlyRunningAlgorithm;

    /**
     * Konstruktor AlgorithmRunnerTask.
     * 
     * @param algorithmList lista algoritama koji se pokreću
     * @param numberOfRuns broj izvršavanja pojedinog algoritma
     * @param mainWindow glavni prozor korisničkog sučelja
     * @param progressBar grafički indikator napretka ukupne zadaće
     * @param startButton gumb za pokretanje izvršavanja zadaće
     * @param stopButton gumb za prekid izvršavanja zadaće
     */
    public AlgorithmRunnerTask(List<Algorithm> algorithmList, int numberOfRuns, MainWindow mainWindow,
	    JProgressBar progressBar, JButton startButton, JButton stopButton, int sawWidth, String problemName, String stockType, String goodCut) {
		this.mainWindow = mainWindow;
		this.algorithmList = algorithmList;
		this.numberOfRuns = numberOfRuns;
		this.progressBar = progressBar;
		this.startButton = startButton;
		this.stopButton = stopButton;
		this.algorithmResultsListeners = new LinkedList<AlgorithmResultsListener>();
		for(Algorithm i : algorithmList){
			try {
				File f = new File("rjesenja\\" + i.getName() + "_" + problemName + ".out");
	    		if (f.exists()){
	    			f.delete();
	    		}
	    		FileWriter ispis = new FileWriter(f);
				ispis.write(stockType + "\n");
				ispis.write(String.valueOf(sawWidth) + "\n");
				ispis.write(goodCut + "\n");
				ispis.flush();
				ispis.close();
				f.setWritable(true, false);
	    		f.setReadable(true, false);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		currentlyRunningTask = this;
	
		currentlyRunningAlgorithm = null;
    }

    /**
     * Pokreće, slijedno izvršava i čeka na završetak izvođenja algoritama. Ažurira grafički
     * indikator napretka ukupne zadaće te zainteresiranim slušačima javlja nove rezultate kako oni
     * pristižu.
     */
    @Override
    protected Void doInBackground() throws Exception {
		int counter = 0;
		for (int i = 0; i < numberOfRuns; ++i) {
		    for (Algorithm a : algorithmList) {
			if (isCancelled()) {
			    return null;
			}
			setProgress((counter * 100) / (numberOfRuns * algorithmList.size()));
			currentlyRunningAlgorithm = a;
			a.run();
			a.waitFor();
			currentlyRunningAlgorithm = null;
			publish(a.getLatestResult());
			++counter;
		    }
		}
		
		return null;
    }

    /**
     * Obrađuje dolazak novih rezultata algoritama tako da obavještava sve zainteresirane slušače.
     */
    @Override
    protected void process(List<AlgorithmResult> results) {
    	fireAlgorithmResultsAvailiable(results);
    }

    /**
     * Obrada završetka zadaće vraćanjem sustava u stanje pripravnosti.
     */
    @Override
    public void done() {
		currentlyRunningTask = null;
		
		Toolkit.getDefaultToolkit().beep();
		mainWindow.setCursor(null);
		startButton.setEnabled(true);
		stopButton.setEnabled(false);
		progressBar.setValue(progressBar.getMinimum());
		progressBar.setString(Config.progressBarReadyText);
		mainWindow.getResultsPanel().getIspisButton().setEnabled(true);
    }

    /**
     * Prekida izvođenje trenutno izvršavane zadaće.
     */
    public void stop() {
		currentlyRunningAlgorithm.kill();
		cancel(true);
    }

    /**
     * Javlja zainteresiranim slušačima kako su novi rezultati algoritama dostupni.
     * 
     * @param results rezultati algoritma
     */
    private void fireAlgorithmResultsAvailiable(List<AlgorithmResult> results) {
		for (AlgorithmResultsListener l : algorithmResultsListeners) {
		    l.algorithmResultsAvailiable(results);
		}
    }

    /**
     * Dodaje slušača zainteresiranog za rezultate algoritama.
     * 
     * @param l slušač zainteresiran za rezultate algoritama
     */
    public void addAlgorithmResultsListener(AlgorithmResultsListener l) {
    	algorithmResultsListeners.add(l);
    }

}
