package hr.fer.zemris.projekt.algorithm;

import java.util.ArrayList;
import java.util.List;

/**
 * Klasa koja predstavlja plan rezanja jednog materijala. 
 * 
 * @author 
 *
 */
public class Layout {
	/** duljina materijala */
	private int stockSize;
	/** veličine itema u planu rezanja materijala */ 
	private List<Integer> itemSizes;
	/** broj itema pojedine veličine */
	private List<Integer> numOfItems;
	/** ostatak bez gubitaka pri rezanju */
	private int remainder;
	/** ostatak sa gubitkom pri rezanju */
	private int totalWaste;
	/** ostatak sa gubitkom pri rezanju */
	private int sawWidth;
	
	/**
	 * Konstruktor koji postavlja duljinu materijala. 
	 * @param stockSize duljina materijala
	 */
	public Layout(int stockSize, int sawWidth) {
		this.stockSize = stockSize;
		this.remainder = stockSize;
		this.totalWaste = stockSize;
		this.sawWidth = sawWidth;
		
		itemSizes = new ArrayList<Integer>();
		numOfItems = new ArrayList<Integer>();
	}
	
	/**
	 * Metoda koja vraća ukupnu duljinu materijala.
	 * @return duljina materijala
	 */
	public int getStockSize() {
		return this.stockSize;
	}
	/**
	 * Metoda koja vraća duljinu upotrebljivog ostatka materijala.
	 * @return upotrebljivi ostatak materijala
	 */
	public int getRemainder() {
		return this.remainder;
	}
	/**
	 * Metoda koja vraća količinu otpada materijala, tj. upotrebljivi ostatak i 
	 * gubitak pri rezanju. 
	 * @return količina otpada materijala
	 */
	public int getWaste() {
		return this.totalWaste;
	}
	/**
	 * Metoda koja pokušava dodati item duljine itemSize u plan rezanja materijala.
	 * @param itemSize duljina itema koji treba rezati
	 * @param cutLoss gubitak pri rezanju
	 * @return true ako je dodavanje uspješno, tj. ima dovoljno mjesta u materijalu, 
	 * 		false inače
	 */
	public boolean addItem(int itemSize) {
		if (itemSize > remainder) return false;
		int index = itemSizes.indexOf(Integer.valueOf(itemSize));
		if (index < 0) {
			itemSizes.add(itemSize);
			numOfItems.add(1);
		} else {
			numOfItems.set(index, numOfItems.get(index) + 1);
		}
		remainder -= itemSize;
		totalWaste -= itemSize;
		
		if(remainder != 0) {
			if (remainder > sawWidth) {
				remainder -= sawWidth;
			} else {
				remainder = 0;
			}
		}
		return true;
	}
	
	/**
	 * Metoda koja vraća veličinu itema koji je na zadanom mjestu u listi.
	 * @param index indeks veličine itema koju treba vratiti
	 * @return veličina itema na mjestu index
	 */
	public int getItem(int index) {
		return itemSizes.get(index);
	}
	
	/**
	 * Metoda koja vraća broj itema određene veličine. 
	 * @param index indeks broja itema koji treba vratiti
	 * @return broj itema za veličinu na indeksu index
	 */
	public int getNumOfItems(int index) {
		return numOfItems.get(index);
	}
	
	/**
	 * Metoda koja vraća broj itema koju su trenutno u planu rezanja materijala.
	 * @return
	 */
	public int numOfItems() {
		int sum = 0;
		for (int i : numOfItems) {
			sum += i;
		}
		return sum;
	}
	
	/**
	 * Metoda koja vraća broj različitih veličina itema u planu rezanja.
	 * @return broj različitih veličina itema
	 */
	public int numOfItemTypes() {
		return itemSizes.size();
	}
	
	/**
	 * Metoda koja ispituje da li je materijal neiskorišten. 
	 * @return true ako je neiskorišten, false inače
	 */
	public boolean isEmpty() {
		return stockSize == remainder;
	}
	
	/**
	 * Metoda koja vraća String koji predstavlja ovaj plan rezanja. 
	 * @return String koji predstavlja plan rezanja
	 */
	public String toString() {
		String ispis = Integer.toString(stockSize) + "\t" + itemSizes.get(0).toString();
		if (!numOfItems.get(0).equals(1)){
			ispis += "*" + numOfItems.get(0).toString();
		}
		for (int i = 1; i < itemSizes.size(); i++) {
			ispis += " + " + itemSizes.get(i).toString();
			if (!numOfItems.get(i).equals(1)){
				ispis += "*" + numOfItems.get(i).toString();
			}
		}
		return ispis;
	}
}
