package hr.fer.zemris.projekt.swing;

import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import hr.fer.zemris.projekt.algorithm.Algorithm;
import hr.fer.zemris.projekt.algorithm.AlgorithmProblem;
import hr.fer.zemris.projekt.algorithm.AlgorithmRunnerTask;
import hr.fer.zemris.projekt.database.DatabaseChangeListener;
import hr.fer.zemris.projekt.database.DatabaseProvider;
import hr.fer.zemris.projekt.util.Config;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

/**
 * Razred predstavlja grafičko sučelje tj. prozor s glavnom funkcionalnošću aplikacije. Nudi
 * postavljanje parametara problema za algoritam te pokretanje, praćenje i prekidanje izvođenja
 * kombinacija algoritama. Također nudi pregled raspoloživih sirovina filtriranih po tipu sirovine i
 * odabir željenih rezova u obliku tablica. Prikazuje i grafičku reprezentaciju rješenja algoritma
 * te međusobnu usporedbu među algoritmima unutar jedne zadaće. Nudi izbornike koji vode do pregleda
 * i uređivanja svih raspoloživih sirovina te tipova sirovina. Također nudi izbornik koji vodi do
 * odabira ulazne datoteke za učitavanje spremljenog problema kao i do odabira izlazne datoteke za
 * spremanje trenutnog problema. Nudi i izbornik koji vodi do općih informacija o programu.
 * 
 * @author
 * 
 */
final public class MainWindow extends JFrame implements DatabaseChangeListener,
	PropertyChangeListener {

    /** Serijski broj */
    private static final long serialVersionUID = 1L;

    /** Nazivi stupaca tablice stockTable */
    private static final String[] stockTableColumnNames = { "Duljina", "Količina", "Tip" };
    /** Nazivi stupaca tablice itemTable */
    private static final String[] itemTableColumnNames = { "Duljina", "Količina" };

    /** Grafička komponenta koja prikazuje rezultate algoritama */
    private ResultsPanel resultsPanel;
    /** Tablica raspoloživih sirovina */
    private JTable stockTable;
    /** Tablica željenih rezova */
    private JTable itemTable;
    /** Padajući izbornik predstavlja izbor tipa sirovine */
    private JComboBox stockTypeFilter;
    /** Mjera za dovoljno dobar rez, ukoliko je unutar te granice rez se smatra savršenim */
    private JFormattedTextField goodEnoughCut;
    /** Debljina pile tj. gubitak materijala pri jednom rezu */
    private JFormattedTextField sawWidth;
    /** Broj izvršavanja pojedinog algoritma */
    private JFormattedTextField numberOfRuns;
    /** Gumb za pokretanje izvršavanja zadaće */
    private JButton startButton;
    /** Gumb za prekid izvršavanja zadaće */
    private JButton stopButton;
    /** Grafički indikator napretka ukupne zadaće */
    private JProgressBar progressBar;
    /** Naziv problema */
    private JTextField problemName;
    /** Prostor za prikaz okvira za izbor */
    private Box algorithmCheckBoxesBox;
    /** Tekst polje za unos duljine željenog reza */
    private JFormattedTextField addItemSize;
    /** Tekst polje za unos količine željenog reza */
    private JFormattedTextField addItemAmount;
    /** Maksimalno vrijeme izvršavanja pojedinog algoritma */
    private JFormattedTextField runTime;
    
    /**
     * Struktura (mapa) koja kao ključ sadrži tip algoritma, a kao vrijednost listu okvira za izbor
     * tog tipa algoritma
     */
    private Map<String, List<JCheckBox>> algorithmCheckBoxes;

    /**
     * Konstruktor StockTypeWindow. Poziva iniciranje grafičkog sučelja tj. prozora kojeg
     * predstavlja razred. Dodaje sebe kao slušača na bazu podataka.
     */
    public MainWindow() {
		initGUI();
	
		DatabaseProvider.getDatabase().addDatabaseChangeListener(this);
    }

    /**
     * Inicira i pokreće grafičko sučelje.
     */
    private void initGUI() {
        setTitle(Config.mainWindowTitle);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                exit();
            }
        });

        Container contentPane = this.getContentPane();
        JMenuBar menuBar = new JMenuBar();
        JMenu menu;
        JMenuItem menuItem;
        JTabbedPane tabbedPane = new JTabbedPane();
        resultsPanel = new ResultsPanel();
        problemName = new JTextField();
        problemName.setBorder(BorderFactory.createTitledBorder(Config.problemNameBorder));
        problemName.setAlignmentX(LEFT_ALIGNMENT);
        problemName.setMaximumSize(new Dimension(99999, 44));
        stockTypeFilter = new JComboBox();
        stockTypeFilter.setOpaque(true);
        stockTypeFilter.setBackground(Color.WHITE);
        stockTypeFilter.setBorder(BorderFactory.createTitledBorder(Config.stockTypeFilterBorder));
        stockTypeFilter.setAlignmentX(LEFT_ALIGNMENT);
        stockTypeFilter.setMaximumSize(new Dimension(99999, 44));
        goodEnoughCut = new JFormattedTextField();
        goodEnoughCut.setBorder(BorderFactory.createTitledBorder(Config.goodEnoughCutBorder));
        goodEnoughCut.setAlignmentY(CENTER_ALIGNMENT);
        goodEnoughCut.setMaximumSize(new Dimension(99999, 44));
        sawWidth = new JFormattedTextField();
        sawWidth.setBorder(BorderFactory.createTitledBorder(Config.sawWidthBorder));
        sawWidth.setAlignmentY(CENTER_ALIGNMENT);
        sawWidth.setMaximumSize(new Dimension(99999, 44));
        algorithmCheckBoxes = new HashMap<String, List<JCheckBox>>();
        algorithmCheckBoxesBox = Box.createVerticalBox();
        algorithmCheckBoxesBox.setOpaque(true);
        algorithmCheckBoxesBox.setBackground(Color.WHITE);
        JScrollPane algorithmCheckBoxesScrollPane = new JScrollPane(algorithmCheckBoxesBox,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        algorithmCheckBoxesScrollPane.setOpaque(true);
        algorithmCheckBoxesScrollPane.setBackground(Color.WHITE);
        algorithmCheckBoxesScrollPane.setBorder(BorderFactory
                .createTitledBorder(Config.algorithmCheckBoxesBorder));
        algorithmCheckBoxesScrollPane.setAlignmentX(LEFT_ALIGNMENT);
        numberOfRuns = new JFormattedTextField();
        numberOfRuns.setBorder(BorderFactory.createTitledBorder(Config.numberOfRunsBorder));
        numberOfRuns.setAlignmentY(BOTTOM_ALIGNMENT);
        numberOfRuns.setMaximumSize(new Dimension(99999, 44));
        runTime = new JFormattedTextField();
        runTime.setBorder(BorderFactory.createTitledBorder(Config.runTimeBorder));
        runTime.setAlignmentY(BOTTOM_ALIGNMENT);
        runTime.setMaximumSize(new Dimension(99999, 44));
        runTime.setValue(new Integer(5));
        startButton = new JButton();
        startButton.setAlignmentY(BOTTOM_ALIGNMENT);
        stopButton = new JButton();
        stopButton.setAlignmentY(BOTTOM_ALIGNMENT);
        progressBar = new JProgressBar();
        progressBar.setAlignmentX(LEFT_ALIGNMENT);
        stockTable = new MyTable(SwingConstants.CENTER);
        itemTable = new MyTable(SwingConstants.CENTER);
        JScrollPane stockTableScrollPane = new JScrollPane(stockTable,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        stockTableScrollPane.setBorder(BorderFactory.createTitledBorder(Config.stockTableBorder));
        stockTableScrollPane.setAlignmentX(LEFT_ALIGNMENT);
        JScrollPane itemTableScrollPane = new JScrollPane(itemTable,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        itemTableScrollPane.setBorder(BorderFactory.createTitledBorder(Config.itemTableBorder));
        itemTableScrollPane.setAlignmentX(LEFT_ALIGNMENT);

        addItemSize = new JFormattedTextField();
        addItemAmount = new JFormattedTextField();
        JButton addItemButton = new JButton();

        menu = new JMenu(Config.menuFile);
        menu.setMnemonic(KeyEvent.VK_Z);

        menuItem = new JMenuItem(Config.menuNew, KeyEvent.VK_N);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                clear();
            }
        });
        menu.add(menuItem);

        menuItem = new JMenuItem(Config.menuOpen, KeyEvent.VK_O);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fileOpen();
            }
        });
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem(Config.menuSave, KeyEvent.VK_S);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fileSave();
            }
        });
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem(Config.menuExit, KeyEvent.VK_X);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                exit();
            }
        });
        menu.add(menuItem);

        menuBar.add(menu);

        menu = new JMenu(Config.menuEdit);
        menu.setMnemonic(KeyEvent.VK_U);

        menuItem = new JMenuItem(Config.menuStock, KeyEvent.VK_R);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                new StockWindow();
            }
        });
        menu.add(menuItem);

        menuItem = new JMenuItem(Config.menuStockType, KeyEvent.VK_T);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                new StockTypeWindow();
            }
        });
        menu.add(menuItem);
        
        menuItem = new JMenuItem(Config.menuReset, KeyEvent.VK_D);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
            	resetDatabase();
            }
        });
        menu.add(menuItem);

        menuBar.add(menu);

        menu = new JMenu(Config.menuInfo);
        menu.setMnemonic(KeyEvent.VK_I);

        menuItem = new JMenuItem(Config.menuAbout, KeyEvent.VK_P);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_DOWN_MASK));
        menuItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                new AboutWindow();
            }
        });
        menu.add(menuItem);

        menuBar.add(menu);

        setJMenuBar(menuBar);

        stockTypeFilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                refreshStockTableFilter();
            }
        });

        startButton.setText(Config.startButtonText);
        startButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                startTask();
            }
        });

        stopButton.setText(Config.stopButtonText);
        stopButton.setEnabled(false);
        stopButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                stopTask();
            }
        });

        progressBar.setStringPainted(true);
        progressBar.setString(Config.progressBarReadyText);

        stockTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        stockTable.setRowSelectionAllowed(true);
        stockTable.setColumnSelectionAllowed(false);
        stockTable.setShowHorizontalLines(true);
        stockTable.setShowVerticalLines(false);
        stockTable.setFillsViewportHeight(true);
        stockTable.setAutoCreateRowSorter(false);
        ToolTipManager.sharedInstance().unregisterComponent(stockTable);
        ToolTipManager.sharedInstance().unregisterComponent(stockTable.getTableHeader());

        itemTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        itemTable.setRowSelectionAllowed(true);
        itemTable.setColumnSelectionAllowed(false);
        itemTable.setShowHorizontalLines(true);
        itemTable.setShowVerticalLines(false);
        itemTable.setFillsViewportHeight(true);
        itemTable.setAutoCreateRowSorter(true);
        ToolTipManager.sharedInstance().unregisterComponent(itemTable);
        ToolTipManager.sharedInstance().unregisterComponent(itemTable.getTableHeader());

        addItemSize.setBorder(BorderFactory.createTitledBorder(Config.itemSizeBorder));
        addItemSize.setMaximumSize(new Dimension(99999, 44));
        addItemAmount.setBorder(BorderFactory.createTitledBorder(Config.itemAmountBorder));
        addItemAmount.setMaximumSize(new Dimension(99999, 44));
        addItemButton.setText(Config.itemButtonText);
        addItemButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
            	addItem();
            	addItemSize.setText(null);
                addItemAmount.setText(null);
            }
        });

        Box mainBox = Box.createHorizontalBox();
        mainBox.setOpaque(true);
        mainBox.setBackground(Color.WHITE);
        Box utilBox = Box.createVerticalBox();
        utilBox.setPreferredSize(new Dimension(250, 0));
        utilBox.setOpaque(true);
        utilBox.setBackground(Color.WHITE);
        utilBox.setAlignmentY(TOP_ALIGNMENT);
        Box paramBox = Box.createVerticalBox();
        paramBox.setOpaque(true);
        paramBox.setBackground(Color.WHITE);
        paramBox.setAlignmentX(LEFT_ALIGNMENT);
        Box runParamBox = Box.createHorizontalBox();
        runParamBox.setOpaque(true);
        runParamBox.setBackground(Color.WHITE);
        runParamBox.setAlignmentX(LEFT_ALIGNMENT);
        Box startBox = Box.createHorizontalBox();
        startBox.setOpaque(true);
        startBox.setBackground(Color.WHITE);
        startBox.setAlignmentX(LEFT_ALIGNMENT);
        Box stockBox = Box.createVerticalBox();
        stockBox.setOpaque(true);
        stockBox.setBackground(Color.WHITE);
        stockBox.setAlignmentY(TOP_ALIGNMENT);
        Box itemBox = Box.createVerticalBox();
        itemBox.setOpaque(true);
        itemBox.setBackground(Color.WHITE);
        itemBox.setAlignmentY(TOP_ALIGNMENT);
        Box addItemBox = Box.createHorizontalBox();
        addItemBox.setOpaque(true);
        addItemBox.setBackground(Color.WHITE);
        addItemBox.setAlignmentX(LEFT_ALIGNMENT);

        utilBox.add(problemName);
        utilBox.add(stockTypeFilter);
        paramBox.add(goodEnoughCut);
        paramBox.add(sawWidth);
        utilBox.add(paramBox);
        utilBox.add(algorithmCheckBoxesScrollPane);
        runParamBox.add(numberOfRuns);
        runParamBox.add(runTime);
        startBox.add(startButton);
        startBox.add(Box.createHorizontalGlue());
        startBox.add(stopButton);
        utilBox.add(Box.createVerticalStrut(44));
        utilBox.add(Box.createVerticalGlue());
        utilBox.add(runParamBox);
        utilBox.add(startBox);
        utilBox.add(Box.createVerticalStrut(5));
        utilBox.add(progressBar);
        stockBox.add(stockTableScrollPane);
        itemBox.add(itemTableScrollPane);
        addItemBox.add(addItemSize);
        addItemBox.add(addItemAmount);
        addItemBox.add(addItemButton);
        itemBox.add(addItemBox);

        mainBox.add(utilBox);
        mainBox.add(stockBox);
        mainBox.add(itemBox);

        tabbedPane.addTab(Config.tabMainText, mainBox);
        tabbedPane.addTab(Config.tabResultsText, resultsPanel);

        contentPane.add(tabbedPane);

        fillAlgorithms();

        clear();
        databaseChanged();

        this.pack();
        setResizable(true);
        setVisible(true);
    } 

    /**
     * Dodaje željen rez.
     */
    private void addItem() {
		int size, amount;
		try {
		    size = Integer.valueOf(addItemSize.getText());
		    amount = Integer.valueOf(addItemAmount.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Duljina i količina reza moraju biti pozitivni cijeli brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (size <= 0 || amount <= 0) {
			JOptionPane.showMessageDialog(this, "Duljina i količina reza moraju biti pozitivni cijeli brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		boolean found = false;
		int r = itemTable.getRowCount();
		for(int i=0; i<r; i++){
			if (((Integer)itemTable.getValueAt(i, 0)).equals(new Integer(size))){
				itemTable.setValueAt(new Integer((Integer)itemTable.getValueAt(i, 1)+amount), i, 1);
				found = true;
				break;
			}
		}
		if (!found){
			((DefaultTableModel) itemTable.getModel()).addRow(new Object[] { size, amount });
		}
    }

    /**
     * Pokreće izvršavanje zadaće.
     */
    private void startTask() {
		List<Algorithm> algorithmList = new LinkedList<Algorithm>();
		for (List<JCheckBox> checkBoxList : algorithmCheckBoxes.values()) {
		    for (JCheckBox checkBox : checkBoxList) {
			if (checkBox.isSelected()) {
			    algorithmList.add(Config.algorithms.get(checkBox.getText()));
			}
		    }
		}
	
		int n, sw;
		try {
		    n = Integer.valueOf(numberOfRuns.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Broj pokretanja mora biti cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		try {
		    sw = Integer.valueOf(sawWidth.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Debljina rezne ploče mora biti cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (algorithmList.isEmpty()) {
			JOptionPane.showMessageDialog(this, "Potrebno je odabrati barem jedan algoritam!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (n <= 0) {
			n = 1;
		}
		if (sw < 0){
			sw = 0;
		}
		AlgorithmProblem algorithmInputData = makeAlgorithmProblem();
		if (algorithmInputData == null || !algorithmInputData.isValid()) {
			JOptionPane.showMessageDialog(this, "Lista zahtjeva za rezanjem prazna ili se izresci ne mogu dobiti iz dostupnih zaliha!", "", JOptionPane.ERROR_MESSAGE);
			return;
		}
		algorithmInputData.exportToAlgorithmInputFile();
	
		startButton.setEnabled(false);
		stopButton.setEnabled(true);
		progressBar.setValue(0);
		progressBar.setString(Config.progressBarWorkingText);
		MainWindow.this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		
		for(Algorithm i : algorithmList){
			i.setProblemName(this.problemName.getText());
		}
		AlgorithmRunnerTask task = new AlgorithmRunnerTask(algorithmList, n, MainWindow.this,
			progressBar, startButton, stopButton, sw, this.problemName.getText(), (String) stockTypeFilter.getSelectedItem(), this.goodEnoughCut.getText());
		task.addPropertyChangeListener(MainWindow.this);
		task.addAlgorithmResultsListener(resultsPanel);
		clearResults();
		task.execute();
    }
    
    /**
     * Stvara i vraća problem za algoritam iz trenutnog stanja grafičkog sučelja.
     * 
     * @return problem za algoritam stvoren iz trenutnog stanja grafičkog sučelja
     */
    private AlgorithmProblem makeAlgorithmProblem() {
		int gdCut, sw, maxTime;
		try {
		    gdCut = Integer.valueOf(goodEnoughCut.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Duljina dobrog reza mora biti cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
		    return null;
		}
		try {
			sw = Integer.valueOf(sawWidth.getText());
			maxTime = Integer.valueOf(runTime.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Maksimalno vrijeme izvođenja mora biti cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
		    return null;
		}
		if (gdCut < 0){
			gdCut = 0;
		}
		if (sw < 0){
			sw = 0;
		}
		if (maxTime < 5){
			maxTime = 5;
		}
		if (stockTypeFilter.getSelectedIndex() == -1 || ((String) stockTypeFilter.getSelectedItem()).isEmpty()) {
			JOptionPane.showMessageDialog(this, "Mora biti odabran tip zaliha koji se koristi!", "", JOptionPane.ERROR_MESSAGE);
		    return null;
		}
		if (problemName.getText().isEmpty()) {
			JOptionPane.showMessageDialog(this, "Naziv problema nije popunjen!", "", JOptionPane.ERROR_MESSAGE);
			return null;
		}
		problemName.setText(problemName.getText().replace(' ', '_'));
		return new AlgorithmProblem(stockTable, itemTable, gdCut, sw,
			(String) stockTypeFilter.getSelectedItem(), problemName.getText(), maxTime);
    }

    /**
     * Prekida izvršavanje trenutno pokrenute zadaće.
     */
    private void stopTask() {
		if (AlgorithmRunnerTask.currentlyRunningTask == null) {
		    return;
		}
		AlgorithmRunnerTask.currentlyRunningTask.stop();
    }

    /**
     * Prikazuje izbornik datoteka. Odabrana datoteka se otvara kao prijašnje spremljen problem te
     * se taj problem onda učitava u grafičko sučelje.
     */
    private void fileOpen() {
		JFileChooser fileChooser = new JFileChooser("./save/");
		int retVal = fileChooser.showOpenDialog(null);
		if (retVal != JFileChooser.APPROVE_OPTION) {
		    return;
		}
		File f = fileChooser.getSelectedFile();
		AlgorithmProblem p = new AlgorithmProblem(f);
		
		loadProblem(p);
    }

    /**
     * U datoteku formata naziv_zadatka.sav se sprema problem kojeg predstavlja
     * trenutno stanje grafičkog sučelja.
     */
    private void fileSave() {
		if (this.problemName.getText().isEmpty()){
			JOptionPane.showMessageDialog(this, "Naziv problema nije popunjen!", "", JOptionPane.ERROR_MESSAGE);
			return;
		}
    	String problemName = this.problemName.getText();
    	problemName = problemName.replace(' ', '_');
		File f = new File("./save/" + problemName + ".sav");
    	if (f.exists()){
    		int izbor = JOptionPane.showConfirmDialog(this, "Problem istog imena već postoji. Prebrisati?", "", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
    		if (izbor == JOptionPane.NO_OPTION){
    			return;
    		}
    		f.delete();
    		
    	}
    	
		AlgorithmProblem p = makeAlgorithmProblem();
		p.exportToProblemSaveFile(f);
    }

    /**
     * Učitava problem u grafičko sučelje te čisti rezultate prijašnjeg problema.
     * 
     * @param p problem koji se učitava
     */
    private void loadProblem(AlgorithmProblem p) {
		if (p.getStockType() != null){
			stockTypeFilter.setSelectedItem(p.getStockType());
		}
		problemName.setText(p.getProblemName());
		goodEnoughCut.setText(String.valueOf(p.getGoodEnoughCut()));
		sawWidth.setText(String.valueOf(p.getSawWidth()));
		Integer[][] itemTableData = p.getItems();
		fillItems(itemTableData);
		clearResults();
    }

    /**
     * Puni strukture podataka (algorithms i algorithmCheckBoxes) koje grafički predstavljaju
     * dostupne i odabrane algoritme.
     */
    private void fillAlgorithms() {
		for (Algorithm a : Config.algorithms.values()) {
		    JCheckBox checkBox = new JCheckBox(a.getName());
		    checkBox.setOpaque(true);
		    checkBox.setBackground(Color.WHITE);
		    List<JCheckBox> checkBoxList = algorithmCheckBoxes.get(a.getType());
		    if (checkBoxList == null) {
			checkBoxList = new LinkedList<JCheckBox>();
			algorithmCheckBoxes.put(a.getType(), checkBoxList);
		    }
		    checkBoxList.add(checkBox);
		}
	
		for (String type : algorithmCheckBoxes.keySet()) {
		    List<JCheckBox> checkBoxList = algorithmCheckBoxes.get(type);
		    Box checkBoxBox = Box.createVerticalBox();
		    checkBoxBox.setOpaque(true);
		    checkBoxBox.setBackground(Color.WHITE);
		    checkBoxBox.setBorder(BorderFactory.createTitledBorder(type));
		    for (JCheckBox checkBox : checkBoxList) {
			checkBoxBox.add(checkBox);
		    }
		    algorithmCheckBoxesBox.add(checkBoxBox);
		}
    }

    /**
     * Puni tablicu željenih rezova predanim podacima.
     * 
     * @param itemTableData podaci kojima se puni tablica željenih rezova
     */
    private void fillItems(Integer[][] itemTableData) {
		TableModel tableModel = new MyTableModel(itemTableData, itemTableColumnNames, true);
		itemTable.setModel(tableModel);
    }

    /**
     * Resetira grafičko sučelje u početno stanje. Prekida trenutno izvršavanu zadaću.
     */
    private void clear() {
		clearFields();
		clearItems();
		clearResults();
		stopTask();
    }

    /**
     * Resetira tekst polja i okvire za izbor algoritama grafičkog sučelja u početno stanje.
     */
    private void clearFields() {
		problemName.setText(Config.defaultProblemName);
		goodEnoughCut.setText(Config.defaultGoodEnoughCut);
		sawWidth.setText(Config.defaultSawWidth);
		for (List<JCheckBox> l : algorithmCheckBoxes.values()) {
		    for (JCheckBox cb : l) {
			cb.setSelected(Config.defaultAlgorithmsSelected);
		    }
		}
		numberOfRuns.setText(Config.defaultNumberOfRuns);
    }

    /**
     * Resetira komponentu s grafičkim prikazom rezultata u početno stanje
     */
    private void clearResults() {
    	resultsPanel.clearResults();
    }

    /**
     * Resetira tablicu s željenim rezovima u početno (prazno) stanje.
     */
    private void clearItems() {
    	fillItems(new Integer[][] {});
    }

    /**
     * Odjavljuje se kao slušač na bazu, prekida trenutno izvršavanu zadaću te gasi aplikaciju.
     */
    private void exit() {
		DatabaseProvider.getDatabase().removeDatabaseChangeListener(this);
		stopTask();
		System.exit(0);
    }

    /**
     * Reagira na promjene napretka trenutno izvršavane zadaće ažuriranjem grafičkog indikatora
     * napretka.
     */
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
		if (AlgorithmRunnerTask.currentlyRunningTask == null
			|| !evt.getPropertyName().equals("progress")) {
		    return;
		}
	
		progressBar.setValue(AlgorithmRunnerTask.currentlyRunningTask.getProgress());
    }
    
    /**
     * Reagira na promjene baze podataka ažuriranjem tablice sirovina i padajućeg izbornika tipova
     * sirovina.
     */
    @Override
    public void databaseChanged() {
		updateStockTypeFilter();
		updateStockTable();
    }

    /**
     * Ažurira padajući izbornik tipova sirovina iz baze podataka.
     */
    private void updateStockTypeFilter() {
		Object[] stockTypes = DatabaseProvider.getDatabase().getStockTypes();
		if (stockTypes != null){
			stockTypeFilter.setModel(new DefaultComboBoxModel(stockTypes));
		}
    }

    /**
     * Ažurira tablicu sirovina iz baze podataka.
     */
    private void updateStockTable() {
		Object[][] stockTableData = DatabaseProvider.getDatabase().getStockData();
		fillStock(stockTableData);
    }

    /**
     * Puni tablicu sirovina predanim podacima te osvježava filter po tipu sirovine.
     * 
     * @param stockTableData podaci kojima se puni tablica sirovina
     */
    private void fillStock(Object[][] stockTableData) {
		TableModel tableModel = new MyTableModel(stockTableData, stockTableColumnNames, false);
		stockTable.setModel(tableModel);
		TableRowSorter<TableModel> rowSorter = new TableRowSorter<TableModel>(tableModel);
		stockTable.setRowSorter(rowSorter);
		refreshStockTableFilter();
    }
    
    /**
     * Osvježava filter po tipu sirovine na tablici sirovina.
     */
    @SuppressWarnings("unchecked")
    private void refreshStockTableFilter() {
		if (stockTypeFilter.getSelectedIndex() == -1) {
		    return;
		}
		((TableRowSorter<TableModel>) stockTable.getRowSorter()).setRowFilter(RowFilter
			.regexFilter("^" + (String) stockTypeFilter.getSelectedItem() + "$", 2));
    }

	public ResultsPanel getResultsPanel() {
		return resultsPanel;
	}
    
	public void resetDatabase(){
		int izbor = JOptionPane.showConfirmDialog(this, "Ova opcija će izbrisati sve podatke o zalihama i tipovima zaliha iz baze. Nastaviti?", "", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		if (izbor == JOptionPane.NO_OPTION){
			return;
		}
		DatabaseProvider.getDatabase().resetDatabase();
	}
}
