package hr.fer.zemris.projekt.swing;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import hr.fer.zemris.projekt.algorithm.AlgorithmResult;
import hr.fer.zemris.projekt.algorithm.AlgorithmResultsListener;
import hr.fer.zemris.projekt.algorithm.Layout;

import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Razred predstavlja grafičku komponentu koja ispisuje rezultate tj. rješenja algoritama.
 * Implementira sučelje AlgorithmResultsListener što joj omogućuje da reagira na pristizanje novih
 * rješenja algoritama.
 * 
 * @author
 * 
 */
public class ResultsPanel extends JPanel implements AlgorithmResultsListener {

    /** Serijski broj */
    private static final long serialVersionUID = 1L;
    
    /** Tekst koji će se koristiti za opis značenja prve vrste stupaca na grafu */
    private static final String CHART_AVG = "Prosjek";
    /** Tekst koji će se koristiti za opis značenja druge vrste stupaca na grafu */
    private static final String CHART_BEST = "Najbolji rezultat";
    /** Duljina svakog materijala na slici */
    private static final int imageMaterialWidth = 15;
    /** Duljina mjerila*/
    private static final int rulerWidth = 25;
    
    /** Lista koja sadrži ime svakog algoritma */
    private List<String> algorithmNames;
    /** Najbolji rezultati za svaki algoritam, gledano prema iskorištenosti */
    private List<AlgorithmResult> bestResultPerAlgorithm;
    /** Lista broja ostvarenih ponavljanja svakog algoritma. 
     * Potrebno za računanje prosjeka. */
    private List<Integer> runsPerAlgorithm;
    /** Slike najboljih rezultata za svaki algoritam */
    private Map<String, BufferedImage> bestResultImages;
    /** Graf usporedbe algoritama */
    private JFreeChart chart;
    /** Skup podataka za crtanje */
    private final DefaultCategoryDataset chartData;
    /** Ime algoritma s najmanjom vrijednosti prosjeka na grafu */
    private String minimumAlg;
    
    /** Panel za crtanje statističke usporedbe algoritama */
    private ChartPanel chartPanel;
    /** Label za ime algoritma */
    private javax.swing.JLabel imeAlgoritmaLabel;
    /** Label za iskorištenost */
    private javax.swing.JLabel iskoristenostLabel;
    /** Gumb za pokretanje ispisa u datoteku */
    private javax.swing.JButton ispisButton;
    /** Područje koje se koristi za prikaz slike rješenja */
    private javax.swing.JScrollPane imageScrollPane;
    private javax.swing.JLabel kolicinaOtpadaLabel;
    /** Label s podcrtanim tekstom Najbolje Rješenje */
    private javax.swing.JLabel najboljeRjesenjeLabel;
    /** Padajući izbornik za odabir algoritma */
    private javax.swing.JComboBox odabirAlgoritmaComboBox;
    /** Label s uputom da se odabere algoritam */
    private javax.swing.JLabel odabirAlgoritmaLabel;
    /** Label za prosječnu iskorištenost */
    private javax.swing.JLabel prosjecnaIskoristenostLabel;
    /** Label za ukupnu količinu materijala iskorištenu */
    private javax.swing.JLabel ukupnoMaterijalaLabel;
    /** Panel koji se koristi kao ikonica svjetlo plave boje. 
     * Koristi se za legendu slike*/
    private javax.swing.JPanel cyanIconPanel;
    /** Panel koji se koristi kao ikonica crvene boje. 
     * Koristi se za legendu slike.*/
    private javax.swing.JPanel redIconPanel;
    /** Panel koji se koristi kao ikonica plave boje. 
     * Koristi se za legendu slike. */
    private javax.swing.JPanel blueIconPanel;
    /** Label koji se koristi za legendu slike.*/
    private javax.swing.JLabel isjecciLabel;
    /** Label koji se koristi za legendu slike. */
    private javax.swing.JLabel ostatakLabel;
    /** Label koji se koristi za legendu slike. */
    private javax.swing.JLabel otpadLabel;
    /** Label koji se koristi za oznaku legende slike. */
    private javax.swing.JLabel legendaSlikeLabel;
    /** CheckBox za oznaku spremanja ostataka u bazu. */
    private javax.swing.JCheckBox spremiOstatkeCheckBox;
    
    /**
     * Konstruktor ResultsPanel.
     */
    public ResultsPanel() {
    	minimumAlg = null;
    	algorithmNames = new ArrayList<String>();
    	chartData = new DefaultCategoryDataset();
    	runsPerAlgorithm = new ArrayList<Integer>();
    	bestResultPerAlgorithm = new ArrayList<AlgorithmResult>();
    	bestResultImages = new TreeMap<String, BufferedImage>();
    	/*sljedeća linija koda stvara novi graf (bar chart), naslov postaljva 
    	 * na prazno, opis kategorija (uz horizontalnu os) postaljva na "Algoritmi",
    	 * opis vrijednost (uz vertikalnu os) postavlja na "Iskorištenost", 
    	 * podatke grafa veže na Dataset na koji referencira varijabla chartData,
    	 * orijentaciju postavlja na okomito, prikazuje legendu, tooltips, ali ne
    	 * i urls */
    	chart = ChartFactory.createBarChart("", "Algoritmi", "Iskorištenost", chartData, 
    			PlotOrientation.VERTICAL, true, true, false);
    	initComponents();
    }
    
    /**
     * Metoda koja inicijalizira sve grafičke komponente ResultsPanel-a
     */
    private void initComponents() {

        chartPanel = new ChartPanel(chart);
        odabirAlgoritmaLabel = new javax.swing.JLabel();
        odabirAlgoritmaComboBox = new javax.swing.JComboBox();
        imeAlgoritmaLabel = new javax.swing.JLabel();
        kolicinaOtpadaLabel = new javax.swing.JLabel();
        ukupnoMaterijalaLabel = new javax.swing.JLabel();
        iskoristenostLabel = new javax.swing.JLabel();
        imageScrollPane = new javax.swing.JScrollPane();
        prosjecnaIskoristenostLabel = new javax.swing.JLabel();
        ispisButton = new javax.swing.JButton();
        najboljeRjesenjeLabel = new javax.swing.JLabel();
        cyanIconPanel = new javax.swing.JPanel();
        redIconPanel = new javax.swing.JPanel();
        blueIconPanel = new javax.swing.JPanel();
        isjecciLabel = new javax.swing.JLabel();
        ostatakLabel = new javax.swing.JLabel();
        otpadLabel = new javax.swing.JLabel();
        legendaSlikeLabel = new javax.swing.JLabel();
        spremiOstatkeCheckBox = new javax.swing.JCheckBox();
        
        imageScrollPane.getVerticalScrollBar().setUnitIncrement(10);
        
        chartPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Statistička usporedba algoritama", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 0, 11), new java.awt.Color(102, 102, 102))); // NOI18N

        javax.swing.GroupLayout chartPanelLayout = new javax.swing.GroupLayout(chartPanel);
        chartPanel.setLayout(chartPanelLayout);
        chartPanelLayout.setHorizontalGroup(
            chartPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 386, Short.MAX_VALUE)
        );
        chartPanelLayout.setVerticalGroup(
            chartPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 194, Short.MAX_VALUE)
        );

        odabirAlgoritmaLabel.setText("Odaberite algoritam čije rješenje želite vidjeti: ");

        odabirAlgoritmaComboBox.setModel(new javax.swing.DefaultComboBoxModel());
        odabirAlgoritmaComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                odabirAlgoritmaComboBoxActionPerformed(evt);
            }
        });

        imeAlgoritmaLabel.setText("Ime algoritma:");

        kolicinaOtpadaLabel.setText("Količina otpada:");

        ukupnoMaterijalaLabel.setText("Ukupna količina materijala:");

        iskoristenostLabel.setText("Iskorištenost:");

        prosjecnaIskoristenostLabel.setText("Prosjecna iskorištenost:");

        ispisButton.setText("Ispis u datoteku i aktualiziranje baze");
        ispisButton.setEnabled(false);
        ispisButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ispisButtonActionPerformed(evt);
            }
        });

        najboljeRjesenjeLabel.setText("<html><u>Najbolje rješenje</u>");

        cyanIconPanel.setBackground(new java.awt.Color(0, 255, 255));

        javax.swing.GroupLayout cyanIconPanelLayout = new javax.swing.GroupLayout(cyanIconPanel);
        cyanIconPanel.setLayout(cyanIconPanelLayout);
        cyanIconPanelLayout.setHorizontalGroup(
            cyanIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );
        cyanIconPanelLayout.setVerticalGroup(
            cyanIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );

        blueIconPanel.setBackground(new java.awt.Color(0, 0, 255));

        javax.swing.GroupLayout blueIconPanelLayout = new javax.swing.GroupLayout(blueIconPanel);
        blueIconPanel.setLayout(blueIconPanelLayout);
        blueIconPanelLayout.setHorizontalGroup(
            blueIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );
        blueIconPanelLayout.setVerticalGroup(
            blueIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );

        isjecciLabel.setText("Isječci");

        ostatakLabel.setText("Ostatak");

        legendaSlikeLabel.setText("Legenda slike: ");

        redIconPanel.setBackground(new java.awt.Color(255, 0, 0));

        javax.swing.GroupLayout redIconPanelLayout = new javax.swing.GroupLayout(redIconPanel);
        redIconPanel.setLayout(redIconPanelLayout);
        redIconPanelLayout.setHorizontalGroup(
            redIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );
        redIconPanelLayout.setVerticalGroup(
            redIconPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );

        otpadLabel.setText("Otpad");

        spremiOstatkeCheckBox.setText("Spremi ostatke u bazu");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(cyanIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(isjecciLabel))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(redIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(otpadLabel))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(blueIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(ostatakLabel))
                            .addComponent(legendaSlikeLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(imageScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 652, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(chartPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addComponent(odabirAlgoritmaLabel)
                                .addGap(4, 4, 4)
                                .addComponent(odabirAlgoritmaComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 88, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                .addGap(61, 61, 61)
                                .addComponent(iskoristenostLabel))
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(ukupnoMaterijalaLabel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                    .addGap(58, 58, 58)
                                    .addComponent(imeAlgoritmaLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                    .addGap(13, 13, 13)
                                    .addComponent(prosjecnaIskoristenostLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                    .addGap(51, 51, 51)
                                    .addComponent(kolicinaOtpadaLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                                    .addGap(46, 46, 46)
                                    .addComponent(najboljeRjesenjeLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(spremiOstatkeCheckBox)
                                .addComponent(ispisButton)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(26, 26, 26)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(odabirAlgoritmaLabel)
                            .addComponent(odabirAlgoritmaComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addComponent(imeAlgoritmaLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(prosjecnaIskoristenostLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(najboljeRjesenjeLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(kolicinaOtpadaLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ukupnoMaterijalaLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(iskoristenostLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(spremiOstatkeCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ispisButton))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(chartPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(legendaSlikeLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(isjecciLabel)
                            .addComponent(cyanIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(ostatakLabel)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(blueIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(2, 2, 2)))
                        .addGap(4, 4, 4)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(redIconPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(otpadLabel)))
                    .addComponent(imageScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 280, Short.MAX_VALUE))
                .addContainerGap())
        );
    }
    
    /**
     * Metoda koja obrađuje klik na gumb za ispis. Treba ispisati trenutno
     * odabrano rješenje u izlaznu datoteku, te aktualizirati stanje baze. 
     * @param evt klik na gumb za ispit
     */
    private void ispisButtonActionPerformed(java.awt.event.ActionEvent evt) {
    	//zatraži odabir datoteke od strane korisnika
    	JFileChooser fileChooser = new JFileChooser();
    	int retVal = fileChooser.showSaveDialog(null);
    	if (retVal != JFileChooser.APPROVE_OPTION) {
    	    return;
    	}
    	File f = fileChooser.getSelectedFile();
    	if (f.getName().indexOf('.')==-1){
            f = new File(f.getAbsolutePath() + ".txt");
    	}
    	if (f.exists()){
    		f.delete();
    	}
    	//saznaj ime algoritma koji je trenutno odabran
    	String algName = (String)odabirAlgoritmaComboBox.getSelectedItem();
        int algIndex = algorithmNames.indexOf(algName);
        
    	bestResultPerAlgorithm.get(algIndex).actualize(f, spremiOstatkeCheckBox.isSelected());
    	retVal = JOptionPane.showOptionDialog(null, 
    			"Želite li spremiti sliku rezultata?", 
    			"Upit spremanja slike",
    			JOptionPane.YES_NO_OPTION, 
    			JOptionPane.QUESTION_MESSAGE,
    			null,
    			new String[]{"Da", "Ne"},
    			"Ne");
    	//Provjerava je li korisnik odabrao DA, ako nije, 
    	if (retVal == JOptionPane.NO_OPTION) {
    		return;
    	}
    	//spremanje slike
    	retVal = fileChooser.showSaveDialog(null);
    	if (retVal != JFileChooser.APPROVE_OPTION) {
    	    return;
    	}
    	f = fileChooser.getSelectedFile();
    	if (!f.getName().endsWith(".pdf")){
            f = new File(f.getAbsolutePath() + ".pdf");
    	}
    	saveImageToPdf(algName, f);
    }
    
   /**
    * Metoda koja sprema sliku najboljeg rezultata algoritma pod dobivenim imenom.
    * @param algName ime algoritma
    * @param file pdf datoteka u koju se sprema slika
    * @return true, ako je spremanje uspjelo, false inače
    */
    private boolean saveImageToPdf(String algName, File file) {
    	Document document = new Document();
		try {
			PdfWriter.getInstance(document, 
				new FileOutputStream(file));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return false;
		} catch (DocumentException e) {
			e.printStackTrace();
			return false;
		}
		document.open();
		
		BufferedImage resultImage = bestResultImages.get(algName);
		BufferedImage temp;
		com.lowagie.text.Image image2;
		
		int pageHeight = (int) (document.getPageSize().getHeight() - document.topMargin() - document.bottomMargin());
		int pageWidth = (int) (document.getPageSize().getWidth() - document.leftMargin() - document.rightMargin());
		int y = rulerWidth;
		int tempHeight = (pageHeight / imageMaterialWidth) * imageMaterialWidth;
		while (tempHeight + rulerWidth > pageHeight) tempHeight -= imageMaterialWidth;
		Graphics2D g;
		
		while (y < resultImage.getHeight() - pageHeight) {
			temp = new BufferedImage(resultImage.getWidth(), tempHeight + rulerWidth, BufferedImage.TYPE_BYTE_INDEXED);
			g = temp.createGraphics();
			//nacrtaj mjerilo
			g.drawImage(resultImage, 0, 0, temp.getWidth(), rulerWidth, 0, 0, resultImage.getWidth(), rulerWidth, null);
			//nacrtaj planove rezanja
			g.drawImage(resultImage, 0, rulerWidth, temp.getWidth(), rulerWidth + tempHeight, 0, y, resultImage.getWidth(), y + tempHeight, null);
			
			try {
				image2 = com.lowagie.text.Image.getInstance(temp, null);
				if (image2.getWidth() > pageWidth) {
					image2.scaleAbsoluteWidth(pageWidth);
				}
				document.add(image2);
				document.newPage();
				
			} catch (BadElementException e) {
				e.printStackTrace();
				return false;
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			} catch (DocumentException e) {
				e.printStackTrace();
				return false;
			}
			y += tempHeight;
		}
		temp = new BufferedImage(resultImage.getWidth(), resultImage.getHeight() - y + rulerWidth, BufferedImage.TYPE_BYTE_INDEXED);
		g = temp.createGraphics();
		//nacrtaj mjerilo
		g.drawImage(resultImage, 0, 0, temp.getWidth(), rulerWidth, 0, 0, resultImage.getWidth(), rulerWidth, null);
		//nacrtaj planove rezanja
		g.drawImage(resultImage, 0, rulerWidth, temp.getWidth(), rulerWidth + tempHeight, 0, y, resultImage.getWidth(), y + tempHeight, null);
		g.dispose();
		try {
			image2 = com.lowagie.text.Image.getInstance(temp, null);
			if (image2.getWidth() > pageWidth) {
				image2.scaleAbsoluteWidth(pageWidth);
			}
			document.add(image2);
			document.newPage();
			
		} catch (BadElementException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		} catch (DocumentException e) {
			e.printStackTrace();
			return false;
		}
		document.close();
		
    	return true;
    }
    
    /**
     * Metoda koja obrađuje odabir algoritma u padajućem izborniku. Osjvežava sve
     * labele, te crta najbolje rješenje tog algoritma. 
     * @param evt odabir novog algoritma preko padajućeg izbornika
     */
    private void odabirAlgoritmaComboBoxActionPerformed(java.awt.event.ActionEvent evt) {
    	JComboBox comboBox = (JComboBox)evt.getSource();
    	if (comboBox.getItemCount() == 0) return;
    	
        String algName = (String)comboBox.getSelectedItem();
        
        updateOutput(algName);
    }
    
    /**
     * Metoda koja na ekran prikazuje podatke vezane uz odabrani algoritam. 
     * @param algName ime odabranog algoritma
     */
    private void updateOutput(String algName) {
    	int index = algorithmNames.indexOf(algName);
    	updateLabels(index);
        BufferedImage image = bestResultImages.get(algName);
        if (image == null) {
        	image = makeImage(index);
        }
        JLabel imageLabel = new JLabel(new ImageIcon(image));
        imageScrollPane.setViewportView(imageLabel);
    }
    
    /**
     * Metoda koja služi za osvježavanje labela sa podacima trenutnog algoritma. 
     * @param index ime algoritma prema kojemu se trebaju osvježiti podaci
     */
    private void updateLabels(int index) {
    	String algName = algorithmNames.get(index);
    	imeAlgoritmaLabel.setText("Ime algoritma: " + algName);
    	
    	prosjecnaIskoristenostLabel .setText("Prosječna iskorištenost: " + 
    			String.format("%1$.2f%%", chartData.getValue(CHART_AVG, algName)) );
    	
    	AlgorithmResult bestResult = bestResultPerAlgorithm.get(index);
    	kolicinaOtpadaLabel.setText("Količina otpada: " + bestResult.getTotalRemainder());
    	ukupnoMaterijalaLabel.setText("Ukupna količina materijala: " + 
    			bestResult.getTotalMaterial());
    	iskoristenostLabel.setText("Iskorištenost: " + 
    			String.format("%1$.2f%%", chartData.getValue(CHART_BEST, algName)) );
    }
    
    /**
     * Metoda koja sve labele vraća u početno stanje. Početno stanje se smatra
     * onim prilikom konstrukcije objekta. 
     */
    private void resetLabels() {
    	imeAlgoritmaLabel.setText("Ime algoritma:");
    	prosjecnaIskoristenostLabel .setText("Prosječna iskorištenost:");
    	kolicinaOtpadaLabel.setText("Količina otpada:");
    	ukupnoMaterijalaLabel.setText("Ukupna količina materijala:");
    	iskoristenostLabel.setText("Iskorištenost:");
    }
    
    /**
     * Prima, obrađuje i prikazuje rezultate tj. rješenja algoritama u toku.
     * @param results lista rezultata koje treba obraditi. 
     */
    public void algorithmResultsAvailiable(List<AlgorithmResult> results) {
		String algName;
		int algIndex;
		int runs;
		double value;
		int roundValue;
		double lowerBound;
		boolean redoAxisRange = false;
		
		for (AlgorithmResult result : results) {
			algName = result.getAlgorithm().getName();
			algIndex = algorithmNames.indexOf(algName);
			if (algIndex < 0) {
				algorithmNames.add(algName);
				runsPerAlgorithm.add(1);
				bestResultPerAlgorithm.add(result);
				
				value = 1.0 - (double) result.getTotalWaste() 
					/ result.getTotalMaterial();
				value *= 100;
				chartData.addValue(value, CHART_AVG, algName);
				chartData.addValue(value, CHART_BEST, algName);
				
				odabirAlgoritmaComboBox.addItem(algName);
			} else {
				//update najboljeg
				value = 1.0 - (double) result.getTotalWaste() 
					/ result.getTotalMaterial();
				value *= 100;
				if (value > (Double)chartData.getValue(CHART_BEST, algName)) {
					chartData.setValue(value, CHART_BEST, algName);
					bestResultPerAlgorithm.set(algIndex, result);
					bestResultImages.put(algName, null);
				}
				
				//update prosjeka
				runs = runsPerAlgorithm.get(algIndex);
				value = (value + (Double)chartData.getValue(CHART_AVG, algName) * runs) 
					/ (runs + 1);
				chartData.setValue(value, CHART_AVG, algName);
				runsPerAlgorithm.set(algIndex, runs + 1);
				
				
			}
			
			//provjeri je li nađen manji minumum
			if (minimumAlg != null && minimumAlg.equals(algName)) {
				String alg;
				double columnValue;
				for (Object columnKey : chartData.getColumnKeys()) {
					alg = (String) columnKey;
					columnValue = (Double) chartData.getValue(CHART_AVG, alg);
					if (columnValue < value) {
						value = columnValue;
						minimumAlg = alg;
					}
				}
				redoAxisRange = true;
			} else if (minimumAlg == null ||
					value < (Double)chartData.getValue(CHART_AVG, minimumAlg)) {
				minimumAlg = algName;
				redoAxisRange = true;
			}
			
			//osvježi granice y-osi
			if (redoAxisRange) {
				value = (Double) chartData.getValue(CHART_AVG, minimumAlg);
				roundValue = (int) (value);
				roundValue = roundValue / 10 * 10 + (roundValue % 10 < 5 ? 0 : 5);
				if ((int) (value * 10) - roundValue * 10 < 5) {
					roundValue -= 5;
				}
				lowerBound = (double) roundValue / 100;
				lowerBound *= 100;
				
				chart.getCategoryPlot().getRangeAxis().setRange(lowerBound, 100.0);
			}
			
			//osvježi podatke algoritma koji je trenutno odabran ako je primljen
			//rezultat njegov
			if (odabirAlgoritmaComboBox.getSelectedItem().equals(algName)) {
				updateOutput(algName);
			}
    	}
    }

    /**
     * Vraća komponentu na početno prazno stanje.
     */
    public void clearResults() {
    	algorithmNames.clear();
    	runsPerAlgorithm.clear();
    	bestResultPerAlgorithm.clear();
    	bestResultImages.clear();
    	for (int i = chartData.getColumnCount() - 1; i >= 0; i--) {
			chartData.removeColumn(i);
		}
    	resetLabels();
    	
    	minimumAlg = null;
    	odabirAlgoritmaComboBox.removeAllItems();
    	ispisButton.setEnabled(false);
    	imageScrollPane.setViewport(null);
    }
    
    /**
     * Metoda koja izrađuje sliku najboljeg rezultata na dobivenom indeksu. 
     * @param index indeks najboljeg rezultata za kojeg treba napraviti sliku
     */
    private BufferedImage makeImage(int index) {
    	AlgorithmResult result = bestResultPerAlgorithm.get(index);
    	
    	int height = imageMaterialWidth;
    	int imgHeight = result.numOfLayouts() * height + rulerWidth;
    	int imgWidth = 750;
    	BufferedImage image = new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_BYTE_INDEXED);
    	Graphics2D graphics = image.createGraphics();
    	graphics.setBackground(Color.white);
    	graphics.clearRect(0, 0, imgWidth, imgHeight);
    	
    	
    	double width;
    	double x = 0;
    	int y = rulerWidth;
    	
    	//crtanje mjerila
    	graphics.setColor(Color.black);
    	//postavlja se početna preciznost na 10cm
    	int preciznost = 100;
    	double increment = ((double) imgWidth / ((double) result.getLongest() / preciznost));
    	//osiguravanje da je minimalni razmak između crtica 4 pixela radi preglednosti
    	while (increment < 5) {
    		preciznost += 100;
    		increment = (double) imgWidth / ((double) result.getLongest() / preciznost);
    	}
    	int counter = 0;
    	for (x = 0; x < imgWidth; x += increment) {
    		if (counter % 5 == 0) {
    			graphics.draw(new Line2D.Double(x, y, x, y - 10));
    			if (counter % 10 == 0) {
    				graphics.drawString(String.valueOf(counter * preciznost), (float)x, y - 15);
    			}
    		} else {
    			graphics.draw(new Line2D.Double(x, y, x, y - 5));
    		}
    		counter++;
    	}
    	
    	//crtanje rezultata
    	Rectangle2D.Double rect;
    	for (Layout layout : result) {
    		x = 0;
    		//iscrtaj svaki item
    		for (int numOfItem = 0; numOfItem < layout.numOfItemTypes(); numOfItem++) {
    			width = (double) layout.getItem(numOfItem) / result.getLongest() * imgWidth;
    			//iscrtaj sve iteme te duljine
    			for (int itemCounter = layout.getNumOfItems(numOfItem) - 1; itemCounter >= 0; itemCounter--) {
    				rect = new Rectangle2D.Double(x, y, width, height);
    				graphics.setColor(Color.cyan);
    				graphics.fill(rect);
    				graphics.setColor(Color.blue);
    				graphics.draw(rect);
    				x += width;
    			}
    		}
    		//iscrtaj ostatak
    		width = (double) layout.getRemainder() / result.getLongest() * imgWidth;
    		rect = new Rectangle2D.Double(x, y, width, height);
    		if (layout.getRemainder() <= result.getGoodCut()) {
    			graphics.setColor(Color.red);
    		} else {
    			graphics.setColor(Color.blue);
    		}
			graphics.fill(rect);
			graphics.setColor(Color.darkGray);
			graphics.draw(rect);
			x += width;
    		
    		y += height;
    	}
    	graphics.dispose();
    	bestResultImages.put(algorithmNames.get(index), image);
    	return image;
    }
    
    /**
     * Metoda koja vraća gumb za aktualiziranje zadatka i ispis u datoteke. 
     * @return ispisButton gumb za aktualiziranje zadatka i ispis u datoteke
     */
	public javax.swing.JButton getIspisButton() {
		return ispisButton;
	}
}
