package hr.fer.zemris.projekt.swing;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import hr.fer.zemris.projekt.database.DatabaseChangeListener;
import hr.fer.zemris.projekt.database.DatabaseProvider;
import hr.fer.zemris.projekt.util.Config;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.TableModel;

/**
 * Razred predstavlja grafičko sučelje tj. prozor s funkcionalnosti pregledavanja raspoloživih
 * tipova sirovina te dodavanjem novih tipova sirovina.
 * 
 * @author
 * 
 */
public class StockTypeWindow extends JFrame implements DatabaseChangeListener {

    /** Serijski broj */
    private static final long serialVersionUID = 1L;

    /** Nazivi stupaca tablice stockTypeTable */
    private static final String[] stockTypeTableColumnNames = { "Naziv", "Širina", "Materijal",
	    "Boja" };

    /** True ako je ovaj prozor otvoren, false ako nije */
    private static boolean windowExists = false;

    /** Tablica tipova sirovina */
    private JTable stockTypeTable;
    /** Naziv tipa sirovine */
    private JTextField stockTypeName;
    /** Širina tipa sirovine */
    private JFormattedTextField stockTypeWidth;
    /** Materijal tipa sirovine */
    private JTextField stockTypeMaterial;
    /** Boja tipa sirovine */
    private JTextField stockTypeColor;

    /**
     * Konstruktor StockTypeWindow. Poziva iniciranje grafičkog sučelja tj. prozora kojeg
     * predstavlja razred. Dodaje sebe kao slušača na bazu podataka.
     */
    public StockTypeWindow() {
		if (windowExists) {
		    setVisible(true);
		}
		windowExists = true;
	
		initGUI();
	
		DatabaseProvider.getDatabase().addDatabaseChangeListener(this);
    }

    /**
     * Inicira i pokreće grafičko sučelje.
     */
    private void initGUI() {
		setTitle(Config.stockTypeWindowTitle);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
		    @Override
		    public void windowClosing(WindowEvent e) {
			DatabaseProvider.getDatabase().removeDatabaseChangeListener(StockTypeWindow.this);
			dispose();
			windowExists = false;
		    }
		});
	
		Container contentPane = getContentPane();
		stockTypeTable = new MyTable(SwingConstants.CENTER);
		stockTypeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		stockTypeTable.setRowSelectionAllowed(true);
		stockTypeTable.setColumnSelectionAllowed(false);
		stockTypeTable.setShowHorizontalLines(true);
		stockTypeTable.setShowVerticalLines(false);
		stockTypeTable.setFillsViewportHeight(true);
		stockTypeTable.setAutoCreateRowSorter(true);
		ToolTipManager.sharedInstance().unregisterComponent(stockTypeTable);
		ToolTipManager.sharedInstance().unregisterComponent(stockTypeTable.getTableHeader());
		JScrollPane stockTableScrollPane = new JScrollPane(stockTypeTable,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		stockTableScrollPane.setBorder(BorderFactory.createTitledBorder(Config.stockTypeWindowTitle));
		stockTableScrollPane.setAlignmentX(LEFT_ALIGNMENT);
		stockTypeName = new JTextField();
		stockTypeName.setBorder(BorderFactory.createTitledBorder(Config.stockTypeNameBorder));
		stockTypeWidth = new JFormattedTextField();
		stockTypeWidth.setBorder(BorderFactory.createTitledBorder(Config.stockTypeWidthBorder));
		stockTypeMaterial = new JTextField();
		stockTypeMaterial.setBorder(BorderFactory
			.createTitledBorder(Config.stockTypeMaterialBorder));
		stockTypeColor = new JTextField();
		stockTypeColor.setBorder(BorderFactory.createTitledBorder(Config.stockTypeColorBorder));
		JButton stockTypeButton = new JButton();
		stockTypeButton.setText(Config.stockTypeButtonText);
		stockTypeButton.addActionListener(new ActionListener() {
		    @Override
		    public void actionPerformed(ActionEvent e) {
			addStockType();
			stockTypeName.setText(null);
            stockTypeWidth.setText(null);
            stockTypeMaterial.setText(null);
            stockTypeColor.setText(null); 
		    }
		});
		JButton stockTypeFromFileButton = new JButton();
        stockTypeFromFileButton.setText(Config.stockTypeFromFileButtonText);
        stockTypeFromFileButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {               
                addStockTypeFromFile();
            }
        });
        JButton exitButton = new JButton();
        exitButton.setText("Gotovo");
        exitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {               
                dispose();
            }
        });
	
		Box stockTypeBox = Box.createVerticalBox();
		stockTypeBox.setOpaque(true);
		stockTypeBox.setBackground(Color.WHITE);
		Box addStockTypeBox = Box.createHorizontalBox();
		addStockTypeBox.setOpaque(true);
		addStockTypeBox.setBackground(Color.WHITE);
		addStockTypeBox.setAlignmentX(LEFT_ALIGNMENT);
		Box addStockTypeButtons = Box.createHorizontalBox();
		addStockTypeButtons.setOpaque(true);
		addStockTypeButtons.setBackground(Color.WHITE);
		addStockTypeButtons.setAlignmentX(LEFT_ALIGNMENT);
		
		addStockTypeBox.add(stockTypeName);
		addStockTypeBox.add(stockTypeWidth);
		addStockTypeBox.add(stockTypeMaterial);
		addStockTypeBox.add(stockTypeColor);
		addStockTypeButtons.add(stockTypeButton);
		addStockTypeButtons.add(stockTypeFromFileButton);
		addStockTypeButtons.add(exitButton);
	
		stockTypeBox.add(stockTableScrollPane);
		stockTypeBox.add(addStockTypeBox);
		stockTypeBox.add(addStockTypeButtons);
	
		contentPane.add(stockTypeBox);
	
		databaseChanged();
	
		pack();
		setResizable(false);
		setLocationRelativeTo(null);
		setVisible(true);
    }

    /**
     * Dodaje specificiran tip sirovine.
     */
    private void addStockType() {
		int width;
		try {
		    width = Integer.valueOf(stockTypeWidth.getText());
		} catch (NumberFormatException ex) {
		    JOptionPane.showMessageDialog(this, "Širina mora biti pozitivan cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (width <= 0){
			JOptionPane.showMessageDialog(this, "Širina mora biti pozitivan cijeli broj!", "", JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (stockTypeName.getText().isEmpty()
			|| stockTypeMaterial.getText().isEmpty() || stockTypeColor.getText().isEmpty()) {
			JOptionPane.showMessageDialog(this, "Sva polja moraju biti popunjena!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		Object[] tipovi = DatabaseProvider.getDatabase().getStockTypes();
		boolean found = false;
		for (int i = 0; i < tipovi.length; i++) {
			if(tipovi[i].equals(stockTypeName.getText())){
				found = true;
				break;
			}
		}
		if (found){
			int izbor = JOptionPane.showConfirmDialog(this, "Već postoji tip tog naziva. Prebrisati?", "", JOptionPane.YES_NO_OPTION);
			if (izbor == JOptionPane.NO_OPTION){
				return;
			}
		}
		DatabaseProvider.getDatabase().removeStockType(stockTypeName.getText());
		DatabaseProvider.getDatabase().insertStockType(stockTypeName.getText(), width,
			stockTypeMaterial.getText(), stockTypeColor.getText());
    }

    /**
     * Reagira na promjene baze podataka ažuriranjem tablice tipova sirovina.
     */
    @Override
    public void databaseChanged() {
    	updateStockTypeTable();
    }

    /**
     * Ažurira tablicu tipova sirovina iz baze podataka.
     */
    private void updateStockTypeTable() {
		Object[][] stockTypeTableData = DatabaseProvider.getDatabase().getStockTypeData();
		if (stockTypeTableData != null){
			fillStockTypes(stockTypeTableData);
		}
    }

    /**
     * Puni tablicu tipova sirovina predanim podacima.
     * 
     * @param stockTypeTableData podaci kojima se puni tablica tipova sirovina
     */
    private void fillStockTypes(Object[][] stockTypeTableData) {
		TableModel tableModel = new MyTableModel(stockTypeTableData, stockTypeTableColumnNames, true);
		stockTypeTable.setModel(tableModel);
    }
    
    /**
     * Učitava listu tipova zaliha iz datoteke.
     */
    private void addStockTypeFromFile() {
    	JFileChooser fileChooser = new JFileChooser();
		int retVal = fileChooser.showOpenDialog(null);
		if (retVal != JFileChooser.APPROVE_OPTION) {
		    return;
		}
		File f = fileChooser.getSelectedFile();
		if (!f.exists()){
			JOptionPane.showMessageDialog(this, "Odabrana datoteka ne postoji!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		try {
			BufferedReader citac = new BufferedReader(new FileReader(f));
			String red = citac.readLine();
			while(red != null){
				String[] temp = red.split("\t");
				stockTypeName.setText(temp[0]);
				stockTypeWidth.setText(temp[1]);
				stockTypeMaterial.setText(temp[2]);
				stockTypeColor.setText(temp[3]);
				this.addStockType();
				red = citac.readLine();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		stockTypeName.setText(null);
        stockTypeWidth.setText(null);
        stockTypeMaterial.setText(null);
        stockTypeColor.setText(null);
    } 

}
