package hr.fer.zemris.projekt.swing;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import hr.fer.zemris.projekt.database.DatabaseChangeListener;
import hr.fer.zemris.projekt.database.DatabaseProvider;
import hr.fer.zemris.projekt.util.Config;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.TableModel;

/**
 * Razred predstavlja grafičko sučelje tj. prozor s funkcionalnosti pregledavanja raspoloživih
 * sirovina te dodavanjem novih sirovina.
 * 
 * @author
 * 
 */
public class StockWindow extends JFrame implements DatabaseChangeListener {

    /** Serijski broj */
    private static final long serialVersionUID = 1L;

    /** Nazivi stupaca tablice stockTypeTable */
    private static final String[] stockTableColumnNames = { "Duljina", "Količina", "Tip" };

    /** True ako je ovaj prozor otvoren, false ako nije */
    private static boolean windowExists = false;

    /** Tablica raspoloživih sirovina */
    private JTable stockTable;
    /** Duljina sirovine */
    private JFormattedTextField stockSize;
    /** Količina sirovine */
    private JFormattedTextField stockAmount;
    /** Naziv tipa sirovine */
    private JComboBox stockTypeName;

    /**
     * Konstruktor StockWindow. Poziva iniciranje grafičkog sučelja tj. prozora kojeg predstavlja
     * razred. Dodaje sebe kao slušača na bazu podataka.
     */
    public StockWindow() {
		if (windowExists) {
		    setVisible(true);
		}
		windowExists = true;
	
		initGUI();
	
		DatabaseProvider.getDatabase().addDatabaseChangeListener(this);
    }

    /**
     * Inicira i pokreće grafičko sučelje.
     */
    private void initGUI() {
		setTitle(Config.stockWindowTitle);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
		    @Override
		    public void windowClosing(WindowEvent e) {
			DatabaseProvider.getDatabase().removeDatabaseChangeListener(StockWindow.this);
			dispose();
			windowExists = false;
		    }
		});
	
		Container contentPane = getContentPane();
		stockTable = new MyTable(SwingConstants.CENTER);
		stockTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		stockTable.setRowSelectionAllowed(true);
		stockTable.setColumnSelectionAllowed(false);
		stockTable.setShowHorizontalLines(true);
		stockTable.setShowVerticalLines(false);
		stockTable.setFillsViewportHeight(true);
		stockTable.setAutoCreateRowSorter(true);
		ToolTipManager.sharedInstance().unregisterComponent(stockTable);
		ToolTipManager.sharedInstance().unregisterComponent(stockTable.getTableHeader());
		JScrollPane stockTableScrollPane = new JScrollPane(stockTable,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		stockTableScrollPane.setBorder(BorderFactory.createTitledBorder(Config.stockTableBorder));
		stockTableScrollPane.setAlignmentX(LEFT_ALIGNMENT);
		stockSize = new JFormattedTextField();
		stockSize.setBorder(BorderFactory.createTitledBorder(Config.stockSizeBorder));
		stockAmount = new JFormattedTextField();
		stockAmount.setBorder(BorderFactory.createTitledBorder(Config.stockAmountBorder));
		stockTypeName = new JComboBox();
		stockTypeName.setOpaque(true);
		stockTypeName.setBackground(Color.WHITE);
		stockTypeName.setBorder(BorderFactory.createTitledBorder(Config.stockTypeNameBorder));
		stockTypeName.setAlignmentX(LEFT_ALIGNMENT);
		JButton stockButton = new JButton();
		stockButton.setText(Config.stockButtonText);
		stockButton.addActionListener(new ActionListener() {
		    @Override
		    public void actionPerformed(ActionEvent e) {
				addStock();
				stockSize.setText(null);
	            stockAmount.setText(null);
		    }
		});
		JButton stockFromFileButton = new JButton();
        stockFromFileButton.setText(Config.stockFromFileButtonText);
        stockFromFileButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                addStockFromFile();
            }
        });
        JButton removeStockButton = new JButton();
        removeStockButton.setText("Briši");
        removeStockButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                removeStock();
                stockSize.setText(null);
	            stockAmount.setText(null);
            }
        });
        JButton exitButton = new JButton();
        exitButton.setText("Gotovo");
        exitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
	
		Box stockBox = Box.createVerticalBox();
		stockBox.setOpaque(true);
		stockBox.setBackground(Color.WHITE);
		Box addStockBox = Box.createHorizontalBox();
		addStockBox.setOpaque(true);
		addStockBox.setBackground(Color.WHITE);
		addStockBox.setAlignmentX(LEFT_ALIGNMENT);
		Box addStockParamBox = Box.createVerticalBox();
		addStockParamBox.setOpaque(true);
		addStockParamBox.setBackground(Color.WHITE);
		Box addStockFieldsBox = Box.createHorizontalBox();
		addStockFieldsBox.setOpaque(true);
		addStockFieldsBox.setBackground(Color.WHITE);
		addStockFieldsBox.setAlignmentX(LEFT_ALIGNMENT);
	
		addStockFieldsBox.add(stockSize);
		addStockFieldsBox.add(stockAmount);
		addStockParamBox.add(addStockFieldsBox);
		addStockParamBox.add(stockTypeName);
	
		addStockBox.add(stockButton);
		addStockBox.add(stockFromFileButton);
		addStockBox.add(removeStockButton);
		addStockBox.add(exitButton);
	
		stockBox.add(stockTableScrollPane);
		stockBox.add(addStockParamBox);
		stockBox.add(addStockBox);
	
		contentPane.add(stockBox);
	
		databaseChanged();
	
		pack();
		setResizable(false);
		setLocationRelativeTo(null);
		setVisible(true);
    }

    /**
     * Dodaje specificiranu sirovinu.
     */
    private void addStock() {
		int size, amount;
		String type = (String) stockTypeName.getSelectedItem();
		try {
		    size = Integer.valueOf(stockSize.getText());
		    amount = Integer.valueOf(stockAmount.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Količina i duljina zaliha moraju biti cijeli brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (stockTypeName.getSelectedIndex() == -1) {
		    return;
		}
		if (size <= 0 || (amount <= 0 && amount != -1)) {
			JOptionPane.showMessageDialog(this, "Količina i duljina zaliha moraju biti pozitivni brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		
		int r = stockTable.getRowCount();
		for(int i=0; i<r; i++){
			int duljina = (Integer) stockTable.getValueAt(i, 0);
			int kolicina = (Integer) stockTable.getValueAt(i, 1);
			Object tip = stockTable.getValueAt(i, 2);
			if (duljina == size && tip.equals(type)){
				if (amount != -1 && kolicina != Integer.MAX_VALUE){
					amount += kolicina;
				} else {
					amount = -1;
				}
				DatabaseProvider.getDatabase().removeStock(duljina, kolicina, (String) tip);
				break;
			}
		}
		DatabaseProvider.getDatabase().insertStock(size, amount, (String) type);
		stockTypeName.setSelectedItem(type);
    }

    /**
     * Reagira na promjene baze podataka ažuriranjem tablice sirovina i padajućeg izbornika tipova
     * sirovina.
     */
    @Override
    public void databaseChanged() {
		updateStockTable();
		updateStockType();
    }

    /**
     * Ažurira tablicu sirovina iz baze podataka.
     */
    private void updateStockTable() {
		Object[][] stockTableData = DatabaseProvider.getDatabase().getStockData();
		if (stockTableData != null){
			fillStock(stockTableData);
		}
    }

    /**
     * Puni tablicu sirovina predanim podacima.
     * 
     * @param stockTableData podaci kojima se puni tablica sirovina
     */
    private void fillStock(Object[][] stockTableData) {
		TableModel tableModel = new MyTableModel(stockTableData, stockTableColumnNames, false);
		stockTable.setModel(tableModel);
    }

    /**
     * Ažurira padajući izbornik tipova sirovina iz baze podataka.
     */
    private void updateStockType() {
		Object[] stockTypes = DatabaseProvider.getDatabase().getStockTypes();
		if (stockTypes != null){
			stockTypeName.setModel(new DefaultComboBoxModel(stockTypes));
		}
    }
    
    /**
     * Učitava listu zaliha iz datoteke.
     */
    private void addStockFromFile() {
    	JFileChooser fileChooser = new JFileChooser();
		int retVal = fileChooser.showOpenDialog(null);
		if (retVal != JFileChooser.APPROVE_OPTION) {
		    return;
		}
		File f = fileChooser.getSelectedFile();
		if (!f.exists()){
			JOptionPane.showMessageDialog(this, "Odabrana datoteka ne postoji!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		try {
			BufferedReader citac = new BufferedReader(new FileReader(f));
			String red = citac.readLine();
			stockTypeName.setSelectedItem(red);
			red = citac.readLine();
			while(red != null){
				String[] temp = red.split("\t");
				stockSize.setText(temp[0]);
				stockAmount.setText(temp[1]);
				this.addStock();
				red = citac.readLine();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		stockSize.setText(null);
        stockAmount.setText(null);
    }
    
    /**
     * Briše označene zalihe iz baze.
     */
    private void removeStock() {
    	int size, amount;
		String type = (String) stockTypeName.getSelectedItem();
		try {
		    size = Integer.valueOf(stockSize.getText());
		    amount = Integer.valueOf(stockAmount.getText());
		} catch (NumberFormatException ex) {
			JOptionPane.showMessageDialog(this, "Količina i duljina zaliha moraju biti cijeli brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		if (stockTypeName.getSelectedIndex() == -1) {
		    return;
		}
		if (size <= 0 || (amount <= 0 && amount != -1)) {
			JOptionPane.showMessageDialog(this, "Količina i duljina zaliha moraju biti pozitivni brojevi!", "", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		
		int r = stockTable.getRowCount();
		for(int i=0; i<r; i++){
			int duljina = (Integer) stockTable.getValueAt(i, 0);
			int kolicina = (Integer) stockTable.getValueAt(i, 1);
			Object tip = stockTable.getValueAt(i, 2);
			if (duljina == size && tip.equals(type)){
				if (amount == -1){
					DatabaseProvider.getDatabase().removeStock(duljina, kolicina, (String) tip);
				} else if (amount != -1 && kolicina != Integer.MAX_VALUE){
					amount = kolicina - amount;
					DatabaseProvider.getDatabase().removeStock(duljina, kolicina, (String) tip);
					if (amount > 0){
						DatabaseProvider.getDatabase().insertStock(size, amount, (String) type);
					}
				}
				break;
			}
		}
		stockTypeName.setSelectedItem(type);
	}
}
